SystemOrganization addCategory: #'Dominion-Cards'!
SystemOrganization addCategory: #'Dominion-AI'!
SystemOrganization addCategory: #Dominion!
SystemOrganization addCategory: #'Dominion-Display'!
SystemOrganization addCategory: #'Dominion-Test'!

Error subclass: #DominionError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!DominionError commentStamp: 'JohnParker 7/29/2012 23:25' prior: 0!
A DominionError is an error that is a Dominion game play error.  It is caught within the core and results in a #command:error message being sent to the player that caused it.
!

OrderedCollection subclass: #CardArea
	instanceVariableNames: 'visibility'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardArea commentStamp: 'JohnParker 7/29/2012 23:20' prior: 0!
A CardArea is a place in a deck that a card can reside.  It subclasses OrderedCollection, but adds the visibility field.

Instance Variables
	visibility:		Symbol

visibility
	- Possible values:
		#global - All players can see this area
		#private - Only the player who owns this deck can see this area
		#hidden - No one can look into this area
!

!CardArea class methodsFor: 'instance creation' stamp: 'JohnParker 7/17/2012 21:15'!
visibility: aSymbol
	^self new visibility: aSymbol.! !

!CardArea methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 21:16'!
visibility 
	^visibility.! !

!CardArea methodsFor: 'initialize-release' stamp: 'JohnParker 7/17/2012 21:45'!
visibility: symbol1
	visibility := symbol1.! !

RectangleMorph subclass: #CardMorph
	instanceVariableNames: 'card label memento remaining count area player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!CardMorph commentStamp: 'MichaelOSullivan 7/30/2012 17:36' prior: 0!
CardMorph is a graphical representation of a Dominion card. It displays itself with the card's label, cost, and number remaining in a pile.!

!CardMorph class methodsFor: 'accessing' stamp: 'JohnParker 7/30/2012 19:07'!
defaultHeight
	^100! !

!CardMorph class methodsFor: 'accessing' stamp: 'JohnParker 7/30/2012 19:07'!
defaultWidth
	^80! !

!CardMorph methodsFor: 'accessing' stamp: 'JohnParker 8/2/2012 12:24'!
area
	^area! !

!CardMorph methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 02:51'!
area: aSymbol
	(aSymbol class inheritsFrom: Symbol) ifFalse: [self halt].
	area := aSymbol.! !

!CardMorph methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/15/2012 16:50'!
cardClass
	^card! !

!CardMorph methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/28/2012 09:09'!
count
	^count! !

!CardMorph methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/27/2012 19:07'!
deselect
	self
		borderColor: Color black;
		borderWidth: self defaultBorderWidth! !

!CardMorph methodsFor: 'initialize' stamp: 'JohnParker 7/30/2012 19:08'!
initialize
	super initialize.
	self width: self class defaultWidth.
	self height: self class defaultHeight.
	self on: #startDrag send: #startDrag:with: to: self.
	memento := CardMemento new.
	memento
		owner: self owner;
		position: self position.
! !

!CardMorph methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/27/2012 19:40'!
mouseUp: evt
	| container playerArea |

	playerArea := self ownerChain detect: [:each | (each class = PlayerArea)] ifNone: [nil].
	playerArea isNil 
		ifTrue: [container := self owner]
		ifFalse: [container := playerArea].
	(evt redButtonChanged
		and: [container class = Kingdom
			or: [container class = Supply
				or: [container class = PlayerArea]]])
		ifTrue: [container select: self]! !

!CardMorph methodsFor: 'structure' stamp: 'MichaelOSullivan 7/24/2012 22:17'!
ownerWas: aClass
	| ownerchain playerArea |
	ownerchain := memento owner ownerChain.
	playerArea := ownerchain select: [:each | each class = PlayerArea].
	^(playerArea size > 0)! !

!CardMorph methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 14:10'!
player
	^player! !

!CardMorph methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 14:10'!
player: aNumber
	(aNumber class inheritsFrom: Number) ifFalse: [self halt].
	player := aNumber.! !

!CardMorph methodsFor: 'printing' stamp: 'JohnParker 8/10/2012 00:21'!
printOn: aStream
	aStream nextPutAll: card cardName.
	card cost ifNotNil: [
		 aStream nextPutAll: '($';
			  print: card cost;
			  nextPutAll: ')'
	].
			  
			  ! !

!CardMorph methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/24/2012 22:47'!
rejectDropMorphEvent: anEvent
	self position: memento position.
	memento owner addMorph: self.! !

!CardMorph methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:33'!
select
	self
		borderColor: Color blue;
		borderWidth: 2! !

!CardMorph methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 11:10'!
setCardType: aCardClass count: aNumber
	| cost |
	
	card := aCardClass.
	count := aNumber.
	self color: card color.
	label := (StringMorph new contents: card cardName).
	label position: (self innerBounds origin) + (5 @ 5).
	self addMorph: label.
	
	card cost ifNotNil: [
		cost := (StringMorph new contents: '$', card cost asString).
		cost position: (self innerBounds origin) + ((self width- (1.5 * cost width)) @ (self height - cost height - 5)).
		self addMorph: cost.
	].

	remaining := StringMorph new contents: aNumber asString.
	remaining position: (self innerBounds origin translateBy: 5 @ (self height - remaining height - 5)).
	self addMorph: remaining.! !

!CardMorph methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/28/2012 09:09'!
setCount: aNumber
	remaining contents: aNumber asString.
	count := aNumber! !

!CardMorph methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/24/2012 21:43'!
startDrag: anItem with: anObject
	memento
		owner: self owner;
		position: self position.
	^ super startDrag: anItem with: anObject! !

!CardMorph methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/25/2012 17:50'!
wantsToBeDroppedInto: aMorph
	^ (aMorph class = PasteUpMorph) not! !

StandardWindow subclass: #GameCreator
	instanceVariableNames: 'scrollPane playerDrops playerTypes cardDrop cardSets cardSet cardChoice'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!GameCreator commentStamp: 'MichaelOSullivan 7/30/2012 17:37' prior: 0!
GameCreator is a dialog that allows a user to select the options for and launch a game of Dominion. The number and type of players and the set of cards used in the match are all configurable.!

!GameCreator methodsFor: 'accessing' stamp: 'JohnParker 7/27/2012 21:00'!
cardSetTypes
	^cardSets keys.! !

!GameCreator methodsFor: 'events' stamp: 'JohnParker 7/30/2012 13:00'!
createGame
	|board deckCount playerNum|
	board := Board new.
	board initialize.
	
	deckCount := playerDrops count: [:each | ((playerTypes at: (each selectedItem)) = nil) not].
	deckCount < 2 ifTrue: [
		self inform: 'You need at least 2 players.'.
		^self.
	].
	
	((playerDrops count: [:each | ((playerTypes at: (each selectedItem)) = GameTable)]) = 1) ifFalse: [
		self inform: 'You need exactly 1 human player'.
		^self.
	].
	
	board decks: deckCount.
	
	board cards: cardSet.
	
	playerNum := 1.
	
	playerDrops do: [:each |
		| player|
		player := (playerTypes at: (each selectedItem)).
		player ifNotNil: [
			player  board: board deck: playerNum.
			playerNum := playerNum + 1.
		]
	].
	
	board startGame.
	! !

!GameCreator methodsFor: 'initialize' stamp: 'JohnParker 8/11/2012 02:24'!
initialize
	| button |
	
	playerTypes := Dictionary new.
	playerTypes at: 'None' put: nil.
	playerTypes at: 'Human' put: GameTable.
	playerTypes at: 'Big Money' put: BigMoney.
	
	cardSets := Dictionary new.
	cardSets at: 'Random' put: #basicRandom.
	cardSets at: 'First Game' put: #basicFirstGame.
	cardSets at: 'Big Money' put: #basicBigMoney.
	cardSets at: 'Interaction' put: #basicInteraction.
	cardSets at: 'Size Distortion' put: #basicSizeDistortion.
	cardSets at: 'Village Square' put: #basicVillageSquare.
	
	super initialize.
	self setLabel: 'Dominion'.
	scrollPane := ScrollPane new.
	self addMorph: scrollPane frame: (0@0 corner: 1@1).
	
	scrollPane addMorph: ('Players' asMorph position: 140@15).
	"Player drop down box"
	playerDrops := OrderedCollection new.
	1 to: 4 do: [:num |
		|player|
		scrollPane addMorph: (('Player ' , num asString) asMorph position: 20@(20+(num*30))).
		
		player := DropListMorph new list: self playerTypes.
		player  position: 80@(15+(num*30)).
		player listSelectionIndex: (self playerTypes indexOf: 'None').
		player getEnabledSelector: #enabled.
		playerDrops add: player.
		scrollPane addMorph: player.
	].
	(playerDrops at: 1) listSelectionIndex: (self playerTypes indexOf: 'Human').
	(playerDrops at: 2) listSelectionIndex: (self playerTypes indexOf: 'Big Money').
	
	
	"Start game button"
	button := (SimpleButtonMorph newWithLabel: 'Start Game') position: 300@400.
	button target: self;
			actionSelector: #createGame;
			color: Color lightGray;
			borderWidth: 3;
			doesBevels;
			borderColor: #raised.
	scrollPane addMorph: button.
	
	
	"Card set drop down box"
	scrollPane addMorph: ('Kingdom Cards' asMorph position: 420@15).
	cardDrop := DropListMorph new list: self cardSetTypes.
	cardDrop position: 380@45.
	cardDrop listSelectionIndex: 1.
	cardDrop addDependent: self.
	scrollPane addMorph: cardDrop.
	cardSet := CardSet new perform: (cardSets at: (cardDrop selectedItem)).
	
	cardChoice := TextMorph new contents: ''.
	cardChoice autoFit: false.
	cardChoice position: 380@100.
	cardChoice extent: 200@200.
	cardChoice borderWidth: 1.
	cardChoice margins: 5.
	scrollPane addMorph: cardChoice.! !

!GameCreator methodsFor: 'accessing' stamp: 'JohnParker 7/27/2012 17:52'!
playerTypes
	^playerTypes keys.! !

!GameCreator methodsFor: 'events' stamp: 'JohnParker 7/28/2012 17:21'!
update: aParameter
	cardSet := CardSet new perform: (cardSets at: (cardDrop selectedItem)).
	cardChoice contents:  (cardSet kingdomCards inject: '' into: [:str :each | str , each cardName , Character lf asString]).
	! !

StandardWindow subclass: #GameTable
	instanceVariableNames: 'board deck children scrollPane selectedCards'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!GameTable commentStamp: 'MichaelOSullivan 7/15/2012 14:08' prior: 0!
GameTable represents an area where players gather to play a game of Dominion. All cards are kept and played on the game table.!

!GameTable class methodsFor: 'instance creation' stamp: 'JohnParker 7/27/2012 17:51'!
board: aBoard deck: aNumber
	self new board: aBoard deck: aNumber.! !

!GameTable methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 23:43'!
board
	^board! !

!GameTable methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 15:50'!
board: aBoard deck: aNumber
	| rect supply turnLogs trashArea playerArea|
	
	board := aBoard.
	deck := (aBoard deck: aNumber).
	children := OrderedCollection new.
	
	selectedCards := OrderedCollection new.
	scrollPane := ScrollPane new.
	self addMorph: scrollPane frame: (0@0 corner: 1@1).
	
	"Supply"
	supply := Kingdom table: self.
	supply setCards: selectedCards.
	children add: supply.
	supply position: -3@0.
	scrollPane scroller addMorph: supply.
	
	
	"Trash Area"
	trashArea := TrashArea table: self.
	trashArea layout: [:previous |
		trashArea bottomRight: (previous bottomRight translateBy: (-10@ -10)).
	].
	supply addChild: trashArea.
	scrollPane scroller addMorph: trashArea.
	
	"Player Area"
	playerArea := PlayerArea table: self.
	playerArea layout: [:previous |
		|newWidth|
		
		playerArea height: (142+142+45+4).
		playerArea left: previous innerBounds right - 2.
		playerArea bottom: previous bottom.
		newWidth  := self width - previous right - 12.
		newWidth > 0 ifTrue: [playerArea width: newWidth].
	].
	supply addChild: playerArea.
	scrollPane scroller addMorph: playerArea.
	
	
	"Competition Area"
	turnLogs := CompetitionArea table: self.
	turnLogs layout: [:previous |
		|newWidth|
		
		turnLogs height: previous innerBounds top.
		turnLogs bottom: previous innerBounds top.
		turnLogs left: previous left.
		newWidth := previous width.
		newWidth := newWidth max: (turnLogs playerAreas size * CompetitorArea minimumWidth).
		newWidth > 0 ifTrue: [turnLogs width: newWidth].
		
		turnLogs playerAreas withIndexDo: [:each :index | 
			each position: (((turnLogs width / turnLogs playerAreas size) * (index - 1)) + turnLogs left+2)@(turnLogs top+2).
			each extent: (((turnLogs width / turnLogs playerAreas size)-4) @ (turnLogs bounds height-4))
		].
	].
	playerArea addChild: turnLogs.
	scrollPane scroller addMorph: turnLogs.
	
	self updateBounds.
	self update.
	self openInWorld. ! !

!GameTable methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/27/2012 19:13'!
buySelectedCards
	selectedCards do: [:each | board execute: (BuyCommand deck: deck number card: each cardClass).
								each deselect].
	selectedCards removeAll! !

!GameTable methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 23:43'!
deck
	^deck! !

!GameTable methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 13:59'!
extent: aPoint
	super extent: aPoint.
	self updateBounds.! !

!GameTable methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/27/2012 23:09'!
gainSelectedCards
	| dict card |
	selectedCards isEmpty ifFalse: [
		dict := Dictionary new.
		card := selectedCards at: 1.
		dict at: #gainCard put: (card cardClass). 
		board execute: (ResolveActionCommand deck: deck number args: dict)
	].
	selectedCards do: [:each | 	each deselect].	
	selectedCards removeAll! !

!GameTable methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/18/2012 22:15'!
initialize
	super initialize.
	self setLabel: 'Dominion'.! !

!GameTable methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 02:31'!
resolveActions
	| dict selectedFromHand selectedFromSupply |
	
	board execute: (ResolveActionCommand deck: deck number args: nil).
	
	"dict := Dictionary new.
	
	(playerArea resolveCard = ThroneRoom) ifTrue: [
		selectedFromHand := playerArea selectedCards collect: [:each | each cardClass].
		selectedFromHand := selectedFromHand select: [:each | each inheritsFrom: ActionCard].
		dict at: #playCard put: selectedFromHand first.
		board execute: (ResolveActionCommand deck: deck number args: dict).
	].

	(playerArea resolveCard = Mine) ifTrue: [
		selectedFromHand := playerArea selectedCards collect: [:each | each cardClass].
		selectedFromHand := selectedFromHand select: [:each | each inheritsFrom: TreasureCard].
		selectedFromSupply := selectedCards collect: [:each | each cardClass].
		selectedFromSupply := selectedFromSupply select: [:each | each inheritsFrom: TreasureCard].
		dict at: #trashCard put: selectedFromHand first.
		dict at: #gainCard put: selectedFromSupply first.
		board execute: (ResolveActionCommand deck: deck number args: dict).
	].

	(playerArea resolveCard = Remodel) ifTrue: [
		selectedFromHand := playerArea selectedCards collect: [:each | each cardClass].
		selectedFromSupply := selectedCards collect: [:each | each cardClass].
		dict at: #trashCard put: selectedFromHand first.
		dict at: #gainCard put: selectedFromSupply first.
		board execute: (ResolveActionCommand deck: deck number args: dict).
	].
	playerArea selectedCards do: [:each | each deselect].
	playerArea selectedCards removeAll.
	selectedCards do: [:each | each deselect].
	selectedCards removeAll"! !

!GameTable methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 14:43'!
scroller
	^scrollPane! !

!GameTable methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 23:43'!
table
	^self! !

!GameTable methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 11:45'!
update
	children do: [:each | each update].! !

!GameTable methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 15:02'!
updateBounds
	children ifNil: [^self].
	children do: [ :each | each updateBounds: each].! !

BorderedMorph subclass: #BoardArea
	instanceVariableNames: 'board deck table children layoutBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!BoardArea class methodsFor: 'instance creation' stamp: 'JohnParker 8/10/2012 23:52'!
table: aGameTable
	^self new table: aGameTable.! !

!BoardArea methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 14:59'!
addChild: aBoardArea
	children add: aBoardArea.! !

!BoardArea methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 23:30'!
board
	^board! !

!BoardArea methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 23:30'!
deck
	^deck! !

!BoardArea methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 17:59'!
layout: aBlock
	layoutBlock := aBlock.! !

!BoardArea methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 23:30'!
table
	^table! !

!BoardArea methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 10:18'!
table: aGameTable
	table := aGameTable.
	board := table board.
	deck := table deck.
	children := OrderedCollection new.
	layoutBlock := [:param| ].
	self borderWidth: 0.
! !

!BoardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:07'!
update
	children ifNil: [^self].
	children do: [ :each | each update].! !

!BoardArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 18:05'!
updateBounds: previous
	layoutBlock value: previous.
	children ifNil: [^self].
	children do: [ :each | each updateBounds: self].! !

BoardArea subclass: #CompetitionArea
	instanceVariableNames: 'playerAreas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!CompetitionArea commentStamp: 'MichaelOSullivan 7/30/2012 18:22' prior: 0!
TurnlogArea is the location on the GameTable that contains the TurnLogs. There is one TurnLogs per player.!

!CompetitionArea methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 18:15'!
playerAreas 
	^playerAreas! !

!CompetitionArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:10'!
rejectDropMorphEvent: anEvent! !

!CompetitionArea methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 10:35'!
table: aGameTable
	super table: aGameTable.
	self borderWidth: 2.
	playerAreas := OrderedCollection new.
	
	1 to: board numDecks do: [:num | 
		|player|
		num = deck number ifFalse: [
			player := CompetitorArea table: table deck: (board deck: num).
			playerAreas add: player.
			self addChild: player.
			self addMorph: player.
		].
	].
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).

	! !

BoardArea subclass: #CompetitorArea
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!CompetitorArea class methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 12:16'!
minimumWidth
	^305! !

!CompetitorArea class methodsFor: 'instance creation' stamp: 'JohnParker 8/11/2012 17:33'!
table: aGameTable deck: aDeck
	^self new table: aGameTable deck: aDeck.! !

!CompetitorArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:08'!
rejectDropMorphEvent: anEvent! !

!CompetitorArea methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 14:21'!
table: aGameTable deck: aDeck
	|turnLogArea title drawArea discardArea revealArea|
	super table: aGameTable.
	deck := aDeck.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	
	title := ('Player ' , deck number asString) asStringMorph.
	self addMorph: title.
	title position: (self position translateBy: (5@5)).
	
	turnLogArea := TurnLog table: table deck: deck.
	turnLogArea layout: [: previous |
		|newWidth|
		turnLogArea topLeft: (previous topLeft translateBy: (5@20)).
		turnLogArea width: (previous width - 10).
		turnLogArea height: 150.
		turnLogArea scrollPane position: turnLogArea topLeft.
		turnLogArea scrollPane extent: turnLogArea extent.
	].
	self addChild: turnLogArea.
	self addMorph: turnLogArea.
	
	"Draw Area"
	drawArea := DrawArea table: table deck: deck.
	drawArea layout: [:previous |
		drawArea topLeft: previous bottomLeft.
	].
	turnLogArea addChild: drawArea.
	self addMorph: drawArea.
	
	"Discard Area"
	discardArea := DiscardArea table: table deck: deck.
	discardArea layout: [:previous |
		discardArea topLeft: previous topRight.
	]. 
	drawArea addChild: discardArea.
	self addMorph: discardArea.
	
	"Reveal Area"
	revealArea := RevealArea table: table deck: deck.
	revealArea layout: [:previous |
		revealArea topLeft: previous topRight.
		revealArea width: 130.
	]. 
	discardArea addChild: revealArea.
	self addMorph: revealArea.
	
	super updateBounds: self.
	
	

	! !

BoardArea subclass: #ControlArea
	instanceVariableNames: 'hand actionLbl actions buyLbl buys fundsLbl funds cleanupButton resolveButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
attack: aCardClass
	|card |
	
	card := deck hand detect: [:each | each class inheritsFrom: ResponseCard] ifNone: [nil].
	card isNil ifFalse: [board execute: (RevealResponseCommand deck: deck number card: card class)].
	
	(aCardClass = Bureaucrat) ifTrue: [
		|command args|
		card := deck hand detect: [:each | each class inheritsFrom: VictoryCard] ifNone: [nil].
		
		args := Dictionary new.
		args at: #bureaucratCard put: nil.
		(card isNil) ifFalse: [
			args at: #bureaucratCard put: card class.
		].
		command := ResolveAttackCommand deck: deck number args: args.
		
		board execute: command.
	].

	(aCardClass = Militia) ifTrue: [
		|dict|
		
		dict := Dictionary new.
		dict at: #discards put: OrderedCollection new.
		1 to: (deck hand size - 3) do: [:num |
			(dict at: #discards) add: (deck hand at: num) class.
		]. 
		board execute: (ResolveAttackCommand deck: deck number args: dict)
	].
	
	((aCardClass = Witch) or: ((aCardClass = Spy) or: (aCardClass = Thief))) ifTrue: [
		board execute: (ResolveAttackCommand deck: deck number args: Dictionary new)
	]! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:08'!
cleanupPressed
	"Expected to be triggered when the Cleanup button is clicked"
	
	board execute: (EndTurnCommand new deck: deck number).
	self update.
	! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
command: aCommand error: aString
	self update.
! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	aNumber = deck number ifFalse: [^self].
	self update! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
deck: aNumber cardMovedPlayer: aCardClass from: area1 to: area2
	self update! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
deckTurnState: aDeckNumber
	self update! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!ControlArea methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 23:03'!
hand
	^hand! !

!ControlArea methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 12:23'!
initialize
	super initialize.
	self extent: 97@130.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	actionLbl := StringMorph new contents: 'Actions:'.
	self addMorph: actionLbl.
	actions := StringMorph new contents: '-1'.
	self addMorph: actions.
	buyLbl := StringMorph new contents: 'Buys:'.
	self addMorph: buyLbl.
	buys := StringMorph new contents: '-1'.
	self addMorph: buys.
	fundsLbl := StringMorph new contents: 'Money:'.
	self addMorph: fundsLbl.
	funds := StringMorph new contents: '-1'.
	self addMorph: funds.
	
	cleanupButton := SimpleButtonMorph new label: 'Cleanup'.
	cleanupButton color: Color lightGray;
					borderWidth: 3;
					doesBevels;
					borderColor: #raised;
					target: self;
					actionSelector: #cleanupPressed.
	self addMorph: cleanupButton.
	
	resolveButton := SimpleButtonMorph new label: 'Resolve'.
	resolveButton color: Color lightGray;
					borderWidth: 3;
					doesBevels;
					borderColor: #raised;
					target: self;
					actionSelector: #resolvePressed.
	self addMorph: resolveButton.
	! !

!ControlArea methodsFor: 'drawing' stamp: 'JohnParker 8/11/2012 23:18'!
render
	| rect |
	actionLbl position: (self innerBounds origin translateBy: 10@10).
	actions topLeft: (actionLbl topRight translateBy: 10@0).
	buyLbl topLeft: (actionLbl bottomLeft translateBy: 0@10).
	buys topLeft: (actions bottomLeft translateBy: 0@10).
	fundsLbl topLeft: (buyLbl bottomLeft translateBy: 0@10).
	funds topLeft: (buys bottomLeft translateBy: 0@10).
	
	cleanupButton topLeft: (fundsLbl bottomLeft translateBy: 0@10).
		
	resolveButton topLeft: (cleanupButton bottomLeft translateBy: 0@10).! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!ControlArea methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 23:03'!
resolveCard
	^resolveCard! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:20'!
resolvePressed
	"Expected to be triggered when the Resolve button is clicked"
	
	board execute: (ResolveActionCommand deck: deck number args: nil).
	self update.
	! !

!ControlArea methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 23:03'!
selectedCards
	^selectedCards! !

!ControlArea methodsFor: 'initialize' stamp: 'JohnParker 8/11/2012 23:12'!
table: aGameTable
	super table: aGameTable.
	board addObserver: self.
	
	self update.
	self render.
	! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 13:39'!
update
	deck isNil ifTrue: [^self].
	
	actions contents: deck actions asString.
	buys contents: deck buys asString.
	funds contents: '$', deck money asString.
	
	super update.! !

!ControlArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:03'!
wantsDroppedMorph: aMorph event: anEvent
	^false! !

BoardArea subclass: #DisplayCardArea
	instanceVariableNames: 'cards cardArea title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

DisplayCardArea subclass: #DiscardArea
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!DiscardArea class methodsFor: 'instance creation' stamp: 'JohnParker 8/11/2012 18:32'!
table: aGameTable deck: aDeck
	^self new table: aGameTable deck: aDeck.! !

!DiscardArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 23:17'!
addCardMorphNonResolve: aMorph
	(aMorph area = #board) ifTrue: [
		board execute: (BuyCommand deck: deck number card: aMorph cardClass).
	].! !

!DiscardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:17'!
area
	^#discard! !

!DiscardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:18'!
card
	deck discardCount > 0 ifTrue: [^deck discardTop] ifFalse: [^nil].! !

!DiscardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:26'!
stackSize
	^deck discardCount! !

!DiscardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:19'!
title
	^'Discard Pile'! !

!DisplayCardArea class methodsFor: 'instance creation' stamp: 'JohnParker 8/12/2012 23:08'!
table: aGameTable deck: aDeck
	^self new table: aGameTable deck: aDeck.! !

!DisplayCardArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 23:10'!
addCardMorphNonResolve: aMorph! !

!DisplayCardArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 23:11'!
addMorph: aMorph
	(aMorph class = CardMorph) ifTrue: [
		board resolving ifTrue: [
			board execute: (MoveCardResolveCommand deck: aMorph player card: aMorph cardClass from: aMorph area to: self area).
		] ifFalse: [
			self addCardMorphNonResolve: aMorph.
		].
		
		((aMorph area = self area) and: (aMorph player = deck number)) ifFalse: [table update].
	] ifFalse: [
		super addMorph: aMorph
	].
	! !

!DisplayCardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:13'!
area
	self subclassResponsibility.! !

!DisplayCardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:13'!
card
	self subclassResponsibility.! !

!DisplayCardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 23:08'!
command: aCommand error: aString
	self update.! !

!DisplayCardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 23:08'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	self update.! !

!DisplayCardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 23:08'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!DisplayCardArea methodsFor: 'geometry' stamp: 'JohnParker 8/12/2012 23:08'!
extent: aPoint
	super extent: aPoint.
	cardArea extent: (aPoint translateBy: -2 @ -15).! !

!DisplayCardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 23:08'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!DisplayCardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:26'!
stackSize
	self subclassResponsibility.! !

!DisplayCardArea methodsFor: 'initailize' stamp: 'JohnParker 8/12/2012 23:15'!
table: aGameTable deck: aDeck
	super table: aGameTable.
	deck := aDeck.
	board addObserver: self.
	board addObserver: self at: deck number. 
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	
	cards := OrderedCollection new.
	cardArea := BorderedMorph new.
	cardArea
		borderInset;
		color: self color;
		position: (self innerBounds origin translateBy: 0@15).
	self addMorph: cardArea.
	self extent: (97@130).
	
	title := StringMorph new contents: self title.
	self addMorph: title.
	title position: self innerBounds origin.! !

!DisplayCardArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:14'!
title
	self subclassResponsibility.! !

!DisplayCardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 23:26'!
update
	|sortedCards cardMorph card |
	cardArea removeAllMorphsIn: cards.
	
	card := self card.
	
	card ifNotNil: [
		cardMorph := CardMorph new setCardType: card count: self stackSize asString.
		cardMorph position: (cardArea innerBounds origin translateBy: 5@5).
		cardMorph area: self area.
		cardMorph player: deck number.
		cards add: cardMorph.
		cardArea addMorph: cardMorph.
	].

	super update.! !

!DisplayCardArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 23:08'!
wantsDroppedMorph: aMorph event: anEvent
	^ (aMorph class = CardMorph)! !

DisplayCardArea subclass: #DrawArea
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!DrawArea class methodsFor: 'instance creation' stamp: 'JohnParker 8/11/2012 18:32'!
table: aGameTable deck: aDeck
	^self new table: aGameTable deck: aDeck.! !

!DrawArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:23'!
area
	^#draw! !

!DrawArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:23'!
card
	^Card! !

!DrawArea methodsFor: 'event handling' stamp: 'JohnParker 8/10/2012 00:16'!
deckShuffled: aNumber	
	self update.! !

!DrawArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:27'!
stackSize
	^deck drawCount.! !

!DrawArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 23:23'!
title
	^'Draw Pile'! !

BoardArea subclass: #HandArea
	instanceVariableNames: 'cards cardArea title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!HandArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 21:33'!
addMorph: aMorph
	aMorph class = CardMorph ifTrue: [
		board resolving ifTrue: [
			board execute: (MoveCardResolveCommand deck: aMorph player card: aMorph cardClass from: aMorph area to: #hand).
		].
		((aMorph area = #hand) and: (aMorph player = deck number)) ifFalse: [table update].
	] ifFalse: [
		super addMorph: aMorph
	].
	
	! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:24'!
command: aCommand error: aString
	self update.! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:24'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	self update.! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:24'!
deck: aNumber cardMovedPrivate: aCardClass from: area1 to: area2
	self update.! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:24'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!HandArea methodsFor: 'geometry' stamp: 'JohnParker 8/12/2012 12:00'!
extent: aPoint
	super extent: aPoint.
	cardArea extent: (aPoint translateBy: -2 @ -15).! !

!HandArea methodsFor: 'initailize' stamp: 'JohnParker 8/12/2012 12:03'!
initialize
	super initialize.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	
	cards := OrderedCollection new.
	cardArea := BorderedMorph new.
	cardArea
		borderInset;
		color: self color;
		position: (self innerBounds origin translateBy: 0@15).
	self addMorph: cardArea.
	self height: 130.
	title := StringMorph new contents: 'Hand'.
	self addMorph: title.
	title position: self innerBounds origin.! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:24'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!HandArea methodsFor: 'initailize' stamp: 'JohnParker 8/11/2012 23:24'!
table: aGameTable
	super table: aGameTable.
	board addObserver: self.
	board addObserver: self at: deck number. ! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:34'!
update
	|pos sortedCards card offset lastMorph|
	pos := cardArea innerBounds origin translateBy: 5@5.
	cardArea removeAllMorphs.
	sortedCards := deck hand asSortedCollection: [:a :b | a cardName < b cardName].
	sortedCards size = 0 ifTrue: [^self].
	
	offset := (((cardArea innerBounds width - CardMorph defaultWidth - 20) / ((sortedCards size - 1) max: 1))) floor - 1.
	offset := (offset min: (CardMorph defaultWidth + 10)) max: 0.
	sortedCards do: [:each |	card := CardMorph new setCardType: each class count: ''.
								card position: pos.
								card area: #hand.
								card player: deck number.
								cardArea addMorph: card.
								pos := pos translateBy: (offset)@0.
								lastMorph := card.].
	lastMorph select.
	super update.! !

!HandArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:25'!
wantsDroppedMorph: aMorph event: anEvent
	^ (aMorph class = CardMorph)! !

BoardArea subclass: #InPlayArea
	instanceVariableNames: 'cards cardArea title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!InPlayArea commentStamp: 'MichaelOSullivan 7/30/2012 17:39' prior: 0!
InPlayArea is a location on the DOminion GameTable where a human player plays his cards. Cards are played as soon as they are dropped onto this area. Cards are rejected by this Morph if it is not the player's turn or if the CardMorph does not originate from the player's hand.
!

!InPlayArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 21:33'!
addMorph: aMorph
	aMorph class = CardMorph ifTrue: [
		board resolving ifTrue: [
			board execute: (MoveCardResolveCommand deck: aMorph player card: aMorph cardClass from: aMorph area to: #play).
		] ifFalse: [
			(aMorph area = #hand) ifTrue: [
				cardArea addMorph: aMorph.
				cards add: aMorph. 
				self playCard: aMorph cardClass.
			].
		].
		((aMorph area = #play) and: (aMorph player = deck number)) ifFalse: [table update].
	] ifFalse: [
		super addMorph: aMorph
	].
	
	! !

!InPlayArea methodsFor: 'event handling' stamp: 'JohnParker 7/30/2012 12:41'!
command: aCommand error: aString
	self update.! !

!InPlayArea methodsFor: 'event handling' stamp: 'JohnParker 7/30/2012 12:39'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	self update.! !

!InPlayArea methodsFor: 'event handling' stamp: 'JohnParker 7/30/2012 12:41'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!InPlayArea methodsFor: 'geometry' stamp: 'JohnParker 8/12/2012 12:04'!
extent: aPoint
	super extent: aPoint.
	cardArea extent: (aPoint translateBy: -2 @ -15).! !

!InPlayArea methodsFor: 'initailize' stamp: 'JohnParker 8/12/2012 12:04'!
initialize
	super initialize.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	cards := OrderedCollection new.
	cardArea := BorderedMorph new.
	cardArea
		borderInset;
		color: self color;
		position: (self innerBounds origin translateBy: 0@15).
	self addMorph: cardArea.
	self height: 130.
	
	title := StringMorph new contents: 'Play Area'.
	self addMorph: title.
	title position: self innerBounds origin.! !

!InPlayArea methodsFor: 'testing' stamp: 'MichaelOSullivan 7/24/2012 22:31'!
isCorrectPhase
	^ deck phase = #Action or: [deck phase = #Treasure]! !

!InPlayArea methodsFor: 'game' stamp: 'MichaelOSullivan 7/25/2012 18:43'!
playCard: aCardClass
	(aCardClass inheritsFrom: TreasureCard) ifTrue: [board execute: (TreasureCommand
																				deck: deck number
																				card: aCardClass)].
	(aCardClass inheritsFrom: ActionCard) ifTrue: [board execute: (ActionCommand
																				deck: deck number
																				card: aCardClass)]! !

!InPlayArea methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/23/2012 21:58'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!InPlayArea methodsFor: 'initailize' stamp: 'JohnParker 8/11/2012 23:36'!
table: aGameTable
	super table: aGameTable.
	
	board addObserver: self.
	board addObserver: self at: deck number. ! !

!InPlayArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:12'!
update
	|pos sortedCards card offset|
	pos := cardArea innerBounds origin translateBy: 5@5.
	cardArea removeAllMorphs.
	sortedCards := deck play asSortedCollection: [:a :b | a cardName < b cardName].
	sortedCards size = 0 ifTrue: [^self].
	
	offset := (((cardArea innerBounds width - CardMorph defaultWidth - 20) / ((sortedCards size - 1) max: 1))) floor - 1.
	offset := (offset min: (CardMorph defaultWidth + 10)) max: 0.
	sortedCards do: [:each |	card := CardMorph new setCardType: each class count: ''.
								card position: pos.
								card area: #play.
								card player: deck number.
								cardArea addMorph: card.
								pos := pos translateBy: (offset)@0 ].
	
	super update.! !

!InPlayArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 23:33'!
wantsDroppedMorph: aMorph event: anEvent
	^ (aMorph class = CardMorph)! !

BoardArea subclass: #Kingdom
	instanceVariableNames: 'title cardMorphs selectedCards'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!Kingdom commentStamp: 'MichaelOSullivan 7/30/2012 17:44' prior: 0!
Kingdom is the location on the Dominion GameTable where the kingdom card piles are stored. Players can select cards from this area to buy or gain.!

!Kingdom methodsFor: 'card management' stamp: 'JohnParker 8/12/2012 14:12'!
addCard: aCardClass
	| card |
	card := (CardMorph new setCardType: aCardClass count: (board cardsRemaining: aCardClass)).
	card area: #board.
	card player: 0.
	cardMorphs add: card.
	cardMorphs sort: [:first :second | self compare: first with: second].
	self addMorph: card.! !

!Kingdom methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:29'!
board
	^board! !

!Kingdom methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/27/2012 17:58'!
buySelectedCards
	selectedCards do: [:each | board execute: (BuyCommand deck: deck number card: each cardClass).
								each 	borderColor: Color black;
										borderWidth: each defaultBorderWidth].
	selectedCards removeAll! !

!Kingdom methodsFor: 'accessing' stamp: 'JohnParker 8/1/2012 12:36'!
cards
	"Returns a collection of all the CardMorphs contained in the Kingdom"
	^cardMorphs! !

!Kingdom methodsFor: 'card management' stamp: 'JohnParker 8/11/2012 10:59'!
compare: first with: second
	|firstStandard secondStandard|
	
	"Cards in the standard set come first"
	firstStandard  := (CardSet standardCards contains: [:each | each = first cardClass]) and: (first cardClass inheritsFrom: CurseCard) not.
	secondStandard  := (CardSet standardCards contains: [:each | each = second cardClass]) and: (second cardClass inheritsFrom: CurseCard) not.
	firstStandard = secondStandard ifFalse: [^firstStandard].
	
	"Rules for ordering within the standard set"
	(firstStandard and: secondStandard) ifTrue: [
		|firstTreasure secondTreasure|
		"Within the standard cards, Treasure cards go first"
		firstTreasure := first cardClass inheritsFrom: TreasureCard.
		secondTreasure := second cardClass inheritsFrom: TreasureCard.
		firstTreasure = secondTreasure ifFalse: [^firstTreasure].
		
		"Treasure and Victory cards in the standard set are sorted by cost"
		^first cardClass cost < second cardClass cost.
	].
	
	"Rules for ordering outside the standard set"
	
	"Cards are ordered first by cost"
	first cardClass cost = second cardClass cost ifFalse: [^first cardClass cost < second cardClass cost].
	
	"Cards with equal cost are ordered by name alphabetically"
	^first cardClass cardName < second cardClass cardName.
	! !

!Kingdom methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/28/2012 09:05'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	self update! !

!Kingdom methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/28/2012 09:05'!
deck: aNumber took: aCardClass
	self update! !

!Kingdom methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 16:54'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!Kingdom methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/18/2012 19:52'!
render
	(KingdomRenderer new initializeWith: self) compute! !

!Kingdom methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/24/2012 22:35'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!Kingdom methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/27/2012 19:15'!
select: aCardMorph
	(selectedCards includes: aCardMorph)
		ifTrue: [selectedCards remove: aCardMorph.
				aCardMorph deselect]
		ifFalse: [selectedCards add: aCardMorph.
				aCardMorph select]! !

!Kingdom methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/25/2012 22:13'!
selectedCards
	^selectedCards! !

!Kingdom methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/27/2012 19:01'!
setCards: aCollection
	selectedCards := aCollection! !

!Kingdom methodsFor: 'initialize' stamp: 'JohnParker 8/1/2012 12:38'!
setDeck: aDeck board: aBoard
	| cardClasses row card |
	"cards stores the CarMorphs as OrderedCollections mapped according to thier cost"
	super initialize.
	board := aBoard.
	deck := aDeck.
	cardMorphs := OrderedCollection new.
	board addObserver: self.
	
	board cardsAvailable do: [:each | self addCard: each].
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	title := StringMorph new contents: 'Kingdom'.
	self addMorph: title.
	self render! !

!Kingdom methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 10:19'!
table: aGameTable
	| cardClasses row card |
	super table: aGameTable.
	self borderWidth: 2.
	"cards stores the CarMorphs as OrderedCollections mapped according to thier cost"
	cardMorphs := OrderedCollection new.
	board addObserver: self.
	
	board cardsAvailable do: [:each | self addCard: each].
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	title := StringMorph new contents: 'Supply'.
	self addMorph: title.
	self render! !

!Kingdom methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:28'!
title
	^title! !

!Kingdom methodsFor: 'drawing' stamp: 'JohnParker 8/12/2012 14:08'!
update	
	self removeAllMorphs.
	cardMorphs removeAll.
	self addMorph: title.
	board cardsAvailable do: [:each | self addCard: each].
	self render.
	
	super update.! !

!Kingdom methodsFor: 'drawing' stamp: 'JohnParker 8/11/2012 15:20'!
updateBounds: previous
	super updateBounds: self.! !

!Kingdom methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/24/2012 22:35'!
wantsDroppedMorph: aMorph event: anEvent
	^false! !

BoardArea subclass: #NotificationArea
	instanceVariableNames: 'log errorLog turn queue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:22'!
attack: aCardClass
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: 'Attacked by ';
		print: aCardClass;
		crlf.
	log contents size = 0 ifTrue: [log contents: stream contents.] ifFalse: [queue addLast: stream contents].
	! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:41'!
command: aCommand error: aString
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: aString;
		crlf.
	errorLog contents: stream contents.! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:41'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage] ifTrue: [errorLog contents: ''].! !

!NotificationArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 15:39'!
errorLog
	^errorLog! !

!NotificationArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 10:06'!
log
	^log! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 09:56'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:45'!
resolutionStep
	queue size = 0 ifTrue: [log contents: ''] ifFalse: [log contents: queue first. queue removeFirst].! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:25'!
resolveAction: aCardClass deck: aNumber
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: 'Please resolve card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ' for deck ';
		nextPutAll: aNumber asString;
		crlf.
	log contents size = 0 ifTrue: [log contents: stream contents.] ifFalse: [queue addLast: stream contents].! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 15:23'!
resolveAction: aCardClass message: aString
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: 'Please resolve card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ': ';
		nextPutAll: aString;
		crlf.
	log contents size = 0 ifTrue: [log contents: stream contents.] ifFalse: [queue addLast: stream contents].! !

!NotificationArea methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 15:42'!
table: aTable
	super initialize.
	super table: aTable.
	
	board addObserver: self.
	board addObserver: self at: deck number. 
	self height: 45.
	
	turn := 1.
	queue := OrderedCollection new.
	self color: Color white.
	log := TextMorph new contents: ''.
	log color: Color white.
	self addMorph: log.
	
	errorLog := TextMorph new contents: ''.
	errorLog color: Color white.
	self addMorph: errorLog.
	errorLog position: (self position translateBy: 0@15).! !

!NotificationArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 09:56'!
wantsDroppedMorph: aMorph event: anEvent
	^false! !

BoardArea subclass: #PlayerArea
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!PlayerArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:12'!
rejectDropMorphEvent: anEvent! !

!PlayerArea methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 15:51'!
table: aGameTable
	|handArea played discardArea drawArea revealArea trashArea controlArea notificationArea|
	super table: aGameTable.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	
	self borderWidth: 2.
	
	"Draw Area"
	drawArea := DrawArea table: table deck: deck.
	drawArea layout: [: previous |
		drawArea bottomLeft: (previous innerBounds bottomLeft translateBy: 5@ -10).
	].
	self addChild: drawArea.
	self addMorph: drawArea.
	
	"Discard Area"
	discardArea := DiscardArea table: table deck: deck.
	discardArea layout: [: previous |
		discardArea bottomLeft: previous bottomRight.
	].
	drawArea addChild: discardArea.
	self addMorph: discardArea.	
	
	"Hand area"
	handArea := HandArea table: table.
	handArea layout: [:previous |
		|newWidth|
		handArea bottomLeft: previous bottomRight.
		newWidth := (table width - handArea left) - 22.
		newWidth > 0 ifTrue: [handArea width: newWidth].
		handArea update.
	].
	discardArea addChild: handArea.
	self addMorph: handArea.
	
	
	"Control Area"
	controlArea := ControlArea table: table.
	controlArea layout: [:previous |
		controlArea bottomLeft: (previous topLeft translateBy: 0@ -5).
	].
	drawArea addChild: controlArea.
	self addMorph: controlArea.
	
	
	"Reveal Area"
	revealArea := RevealArea table: table deck: deck.
	revealArea layout: [:previous |
		revealArea bottomLeft: previous bottomRight.
		revealArea width: ((CardMorph defaultWidth * revealArea cards size) max: 97).
	].
	controlArea addChild: revealArea.
	self addMorph: revealArea.
	
	"Play area"
	played := InPlayArea table: table.
	played layout: [:previous |
		|newWidth|
		played bottomLeft: previous bottomRight.
		newWidth := (table width - played left) - 22.
		newWidth > 0 ifTrue: [played width: newWidth].
		played update.
	].
 	revealArea addChild: played.
	played extent: ((handArea topRight x - revealArea bottomRight x)@handArea height).
	self addMorph: played.
	
	"Notification Area"
	notificationArea := NotificationArea table: table.
	notificationArea layout: [:previous |
		|newWidth|
		
		notificationArea topLeft: previous innerBounds topLeft.
		newWidth := table width - previous left - 20.
		newWidth > 0 ifTrue: [notificationArea width: newWidth].
		
		notificationArea log position: (notificationArea topLeft translateBy: 5@5).
		notificationArea log extent: notificationArea extent.
	].
	self addChild: notificationArea.
	self addMorph: notificationArea.
	
	super updateBounds: self.
	
	

	! !

BoardArea subclass: #RevealArea
	instanceVariableNames: 'cards cardArea title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!RevealArea class methodsFor: 'instance creation' stamp: 'JohnParker 8/11/2012 18:33'!
table: aGameTable deck: aDeck
	^self new table: aGameTable deck: aDeck.! !

!RevealArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 21:32'!
addMorph: aMorph
	aMorph class = CardMorph ifTrue: [
		board resolving ifTrue: [
			board execute: (MoveCardResolveCommand deck: aMorph player card: aMorph cardClass from: aMorph area to: #reveal).
		] ifFalse: [
		].
		((aMorph area = #reveal) and: (aMorph player = deck number)) ifFalse: [table update].
	] ifFalse: [
		super addMorph: aMorph
	].
	
	! !

!RevealArea methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 14:18'!
cards
	^cards! !

!RevealArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 11:12'!
command: aCommand error: aString
	self update.! !

!RevealArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 11:12'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	self update.! !

!RevealArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 11:12'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!RevealArea methodsFor: 'geometry' stamp: 'JohnParker 8/12/2012 11:56'!
extent: aPoint
	super extent: aPoint.
	cardArea extent: (aPoint translateBy: -2 @ -15).! !

!RevealArea methodsFor: 'initailize' stamp: 'JohnParker 8/12/2012 11:57'!
initialize
	super initialize.
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	cards := OrderedCollection new.
	cardArea := BorderedMorph new.
	cardArea
		borderInset;
		color: self color;
		position: (self innerBounds origin translateBy: 0@15).
	self addMorph: cardArea.
	self extent: (97@130).
	
	title := StringMorph new contents: 'Reveal Area'.
	self addMorph: title.
	title position: self innerBounds origin.! !

!RevealArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 11:12'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!RevealArea methodsFor: 'initailize' stamp: 'JohnParker 8/11/2012 18:31'!
table: aGameTable deck: aDeck
	super table: aGameTable.
	deck := aDeck.
	board addObserver: self.
	board addObserver: self at: deck number. ! !

!RevealArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:12'!
update
	|pos sortedCards card offset|
	pos := cardArea innerBounds origin translateBy: 5@5.
	self removeAllMorphsIn: cards.
	sortedCards := deck reveal asSortedCollection: [:a :b | a cardName < b cardName].
	sortedCards size = 0 ifTrue: [^self].
	
	offset := (((cardArea innerBounds width - 20) / sortedCards size) - CardMorph defaultWidth) floor - 1.
	offset := offset min: 10.
	sortedCards do: [:each |	card := CardMorph new setCardType: each class count: ''.
								card position: pos.
								cards add: card.
								card area: #reveal.
								card player: deck number.
								super addMorph: card.
								pos := pos translateBy: (card width + offset)@0 ].
	
	super update.
	! !

!RevealArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 11:14'!
wantsDroppedMorph: aMorph event: anEvent
	^aMorph class = CardMorph
! !

BoardArea subclass: #TrashArea
	instanceVariableNames: 'cards cardArea title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!TrashArea methodsFor: 'submorphs-add/remove' stamp: 'JohnParker 8/12/2012 21:32'!
addMorph: aMorph

	(aMorph class = CardMorph) ifTrue: [
		board resolving ifTrue: [
			board execute: (MoveCardResolveCommand deck: aMorph player card: aMorph cardClass from: aMorph area to: #trash).
		].
		
		((aMorph area = #trash) and: (aMorph player = 0)) ifFalse: [table update].
	] ifFalse: [
		super addMorph: aMorph
	].
	! !

!TrashArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 12:02'!
command: aCommand error: aString
	self update.! !

!TrashArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 12:02'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	self update.! !

!TrashArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 12:02'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!TrashArea methodsFor: 'geometry' stamp: 'JohnParker 8/11/2012 23:59'!
extent: aPoint
	cardArea extent: (aPoint translateBy: -8 @ -8).
	super extent: aPoint.! !

!TrashArea methodsFor: 'initailize' stamp: 'JohnParker 8/12/2012 00:00'!
initialize
	super initialize.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	cards := OrderedCollection new.
	cardArea := BorderedMorph new.
	cardArea
		borderInset;
		color: self color;
		position: (self innerBounds origin translateBy: 2@2).
	self addMorph: cardArea.
	
	title := StringMorph new contents: 'Trash'.
	self addMorph: title.
	title position: (self innerBounds origin translateBy: 25@50).! !

!TrashArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 12:02'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!TrashArea methodsFor: 'initailize' stamp: 'JohnParker 8/11/2012 14:49'!
table: aGameTable
	super table: aGameTable.
	board addObserver: self.
	self extent: (100@120).! !

!TrashArea methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:13'!
update
	|sortedCards cardMorph card |
	cardArea removeAllMorphsIn: cards.
	
	card := board trashTop.
	
	card ifNotNil: [
		cardMorph := CardMorph new setCardType: card class count: board trashCount asString.
		cardMorph position: (cardArea innerBounds origin translateBy: 2@2).
		cardMorph area: #trash.
		cardMorph player: 0.
		cards add: cardMorph.
		cardArea addMorph: cardMorph.
	].
	
	super update.! !

!TrashArea methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 12:02'!
wantsDroppedMorph: aMorph event: anEvent
	^ (aMorph class = CardMorph)! !

BoardArea subclass: #TurnLog
	instanceVariableNames: 'log turn scrollPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!TurnLog commentStamp: 'MichaelOSullivan 7/30/2012 18:21' prior: 0!
TurnLog is a custom scroll area that prints information on events that occur during the course of the game.!

!TurnLog class methodsFor: 'instance creation' stamp: 'JohnParker 8/11/2012 17:52'!
table: aTable deck: aDeck
	^self new table: aTable deck: aDeck.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 14:59'!
attack: aCardClass
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Attacked by ';
		print: aCardClass;
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 17:18'!
command: aCommand error: aString
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: aString;
		crlf.
	log contents: stream contents.
	scrollPane isAScrollbarShowing ifTrue: [scrollPane scrollBy: 0 @ (-1 * scrollPane vTotalScrollRange)].! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:50'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	| stream |
	deck number = aNumber ifFalse: [^self].
	
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' moved card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ' from ';
		nextPutAll: area1 asString;
		nextPutAll: ' to ';
		nextPutAll: area2 asString;
		crlf.
	
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:50'!
deck: aNumber cardMovedPlayer: aCardClass from: area1 to: area2
	| stream |
	table deck = aNumber ifFalse: [^self].
	
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' moved card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ' from ';
		nextPutAll: area1 asString;
		nextPutAll: ' to ';
		nextPutAll: area2 asString;
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:50'!
deck: aNumber took: aCardClass
	| stream |
	deck number = aNumber ifFalse: [^self].
	
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' took card ';
		nextPutAll: aCardClass cardName;
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:50'!
deck: aNumber trashed: aCardClass
	| stream |
	deck number = aNumber ifFalse: [^self].
	
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' trashed ';
		nextPutAll: aCardClass cardName;
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:51'!
deckShuffled: aNumber
	| stream |
	deck number = aNumber ifFalse: [^self].
	
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Deck ';
		print: aNumber;
		nextPutAll: ' was shuffled.';
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/11/2012 16:55'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:51'!
gameOverWinners: deckNumbers
	| stream winnerString|
	
	deckNumbers size = 1 ifTrue: [
		winnerString := 'Player ' , deckNumbers first asString.
	] ifFalse: [
		winnerString := 'Players '.
		deckNumbers do: [:each |
			winnerString := winnerString , each asString.
			winnerString := winnerString , (deckNumbers last = each ifFalse: [', '] ifTrue: [' ']).
			winnerString := winnerString , ((deckNumbers at: (deckNumbers size - 1)) = each ifFalse: [''] ifTrue: ['and ']).
		]
	].

	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: winnerString;
		nextPutAll: ' won!!.';
		crlf;
		nextPutAll: 'Scores:';
		crlf.
		(1 to: board numberOfPlayers) do: [:i | stream
												nextPutAll: 'Player ';
												print: i;
												nextPutAll: ': ';
												print: ((board deck: i) victoryPoints); 
												crlf].
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/24/2012 22:36'!
repelsMorph: aMorph event: ev
	^(self wantsDroppedMorph: aMorph event: ev) not! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:51'!
resolveAction: aCardClass deck: aNumber
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Please resolve card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ' for deck ';
		nextPutAll: aNumber asString;
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:51'!
resolveAction: aCardClass message: aString
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Please resolve card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ': ';
		nextPutAll: aString;
		crlf.
	log contents: stream contents.
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 18:38'!
scrollPane 
	^scrollPane! !

!TurnLog methodsFor: 'initialize' stamp: 'JohnParker 8/12/2012 11:47'!
table: aTable deck: aDeck
	super initialize.
	super table: aTable.
	
	deck := aDeck.
	board addObserver: self.
	board addObserver: self at: deck number. 
	
	scrollPane := GeneralScrollPane new.
	self addMorph: scrollPane.
	turn := 1.
	log := TextMorph new contents: ''.
	log autoFit: true.
	scrollPane scroller addMorph: log.
	scrollPane extent: self extent.
	scrollPane position: self position.
	scrollPane scrollTarget: log.
	scrollPane color: Color white.! !

!TurnLog methodsFor: 'event handling' stamp: 'JohnParker 8/12/2012 11:51'!
turnStarted: aNumber
	| stream |
	deck number = aNumber ifTrue: [
		stream := String new writeStream.
		stream
			nextPutAll: log contents;
			crlf;
			nextPutAll: 'Player ';
			print: aNumber;
			nextPutAll: ': Turn #';
			print: turn;
			crlf;
			crlf.
		turn := turn + 1.
		log contents: stream contents].
	scrollPane vScrollbarValue: 1.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/24/2012 22:24'!
wantsDroppedMorph: aMorph event: anEvent
	^false! !

TestCase subclass: #AITest
	instanceVariableNames: 'board deck1 deck2 player1 player2 AI1 AI2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!AITest methodsFor: 'initialize-release' stamp: 'JohnParker 7/29/2012 20:44'!
setupGame
	board := Board new.
	board initialize.
	board decks: 2.
	deck1 := board deck: 1.
	deck2 := board deck: 2.
	AI1 := BigMoneyStrategy board: board deck: 1. 
	AI2 := BigMoneyStrategy board: board deck: 2. ! !

!AITest methodsFor: 'testAttack' stamp: 'JohnParker 7/29/2012 20:38'!
testBureaucrat
	
	|cardsTodiscard hand|
	
	self setupGame.
	AI2 attack: Bureaucrat.
	
	cardsTodiscard := AI2 testingResult.
	hand := deck2 hand.
     (deck1 handContains: Estate) ifTrue: [ 	self assert: ((cardsTodiscard at: #bureaucratCard) = Estate). ]
								  ifFalse: [ self assert: ((cardsTodiscard at: #bureaucratCard) = nil). ].

	! !

!AITest methodsFor: 'testFull' stamp: 'JohnParker 7/30/2012 12:49'!
testFullGame
	self setupGame.
	
	board cards: CardSet new basicFirstGame.
	board startGame.
	
	self assert: board gameOver = true.! !

!AITest methodsFor: 'testAttack' stamp: 'ChrisLau 7/29/2012 18:14'!
testMilitia
	"A new hand has 2 Estate and 3 Coppers. When attacked by Militia, it will discard two Estate "
	|cardsTodiscard|
	
	self setupGame.
	AI2 attack: Militia.
	cardsTodiscard := AI2 testingResult.

	 (deck2 handContains: Estate) ifTrue: [self assert:  (cardsTodiscard at: 1) = Estate].
	self assert:  (((cardsTodiscard at: 2) = Estate) or: ((cardsTodiscard at: 2) = Copper)).
	! !

!AITest methodsFor: 'testBuy' stamp: 'ChrisLau 7/29/2012 15:29'!
testTurnedStart
	"test the decision making strategy for buying card, playing treasure and playing action"
	"A new hand will contain either 3 or 4 dolloar, so the AI will only buy a silver for the first turn "
	|cardsToBuy |
	self setupGame.
	"Treasure Phase"
	AI1 playTreasures.
	
	"Buy Phase"
	cardsToBuy := AI1 CardsToBuy.
	self assert: ((cardsToBuy at:1) = 0).
	self assert: ((cardsToBuy at:2) = 0).
	self assert: ((cardsToBuy at:3) = 0).
	self assert: ((cardsToBuy at:4) = 0).
	self assert: ((cardsToBuy at:5) = 1).
! !

TestCase subclass: #CoreTest
	instanceVariableNames: 'board deck1 deck2 player1 player2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!CoreTest methodsFor: 'private' stamp: 'JohnParker 7/27/2012 18:03'!
setupGame
	board := Board new.
	board initialize.
	board decks: 2.
	deck1 := board deck: 1.
	deck2 := board deck: 2.

	player1 := TestPlayer new.
	player1 board: board deck: 1.
	player2 := TestPlayer new.
	player2 board: board deck: 2.
	
	board startGame.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:34'!
testActionCommand
	self setupGame.
	
	"Send an action command to the wrong deck.  Make sure it returns an error."
	board execute: (ActionCommand deck: 2 card: Smithy).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send an action command on a non action card.  Make sure it returns an error."
	board execute: (ActionCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	"Send an action command after the action phase is complete.  Make sure it returns an error."
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:01'!
testAdventurer
	|treasure|
	self setupGame.
	
	self assert: Adventurer cost = 6.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	treasure := (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size.
	deck1 z_addCardToHand: Adventurer.
	board execute: (ActionCommand deck: 1 card: Adventurer).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size = (treasure + 2).
	self assert: (deck1 play at: 1) class = Adventurer.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:02'!
testBureaucrat
	self setupGame.
	
	self assert: Bureaucrat cost = 4.
	
	"Play a Bureaucrat.  Make sure that a silver is added to the top of the draw and that other players either placed a victory card on top of the draw or have no victory cards."
	deck1 z_addCardToHand: Bureaucrat.
	board execute: (ActionCommand deck: 1 card: Bureaucrat).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_draw last class = Silver.
	self assert: (((deck2 hand size = 4) and: (deck2 z_draw last class = Estate)) or: ((deck2 hand select: [:each | each class = Copper]) size = 5)).
	self assert: (deck1 play at: 1) class = Bureaucrat.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/26/2012 12:38'!
testBuyCommand
	self setupGame.
	board addCard: Adventurer.
	
	"Make sure the phase of each deck is correct.  Deck 1 should be at Action since that is the first phase of a turn.  Deck 2 should be at None because it is not Deck 2s turn."
	self assert: deck1 phase = #Action.
	self assert: deck2 phase = #None.
	
	"Send a buy command to the correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	board execute: (TreasureCommand deck: 1 card: Copper).
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: deck1 money = 2.
	deck1 addBuys: 1.
	board execute: (BuyCommand deck: 1 card: Estate).
	self assert: player1 errorCommand isNil.
	self assert: deck1 z_discard size = 1.
	self assert: deck1 money = 0.
	self assert: (deck1 z_discard at: 1) class = Estate.
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: deck1 z_discard size = 7.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.
	
	"Send a buy command to the wrong deck.  Make sure it returns an error."
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send a buy command to the new correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	board execute: (BuyCommand deck: 2 card: Copper).
	self assert: player2 errorCommand isNil.
	self assert: deck2 z_discard size = 6.
	self assert: (deck1 z_discard at: 2) class = Copper.
	self assert: deck2 phase = #None.
	self assert: deck1 = board currentDeck.
	
	board execute: (BuyCommand deck: 1 card: Adventurer).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	self assert: deck1 z_discard size = 7.
	self assert: deck1 phase = #Action.
	self assert: deck1 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:25'!
testCellar
	| victory | 
	self setupGame.
	
	self assert: Cellar cost = 2.
	
	"Play a Cellar.  We are using a default player who will discard all of his victory cards, so make sure he does so and that the correct number of cards are drawn to replace them."
	victory := (deck1 hand select: [:each | each class inheritsFrom: VictoryCard]) size.
	deck1 z_addCardToHand: Cellar.
	board execute: (ActionCommand deck: 1 card: Cellar).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = victory).
	self assert: ((1 to: victory) select: [:n | (deck1 z_discard at: n) class = Estate]) size = victory.
	self assert: (deck1 play at: 1) class = Cellar.
	self assert: deck1 phase = #Action.
	self assert: deck1 actions = 1.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:25'!
testChancellor
	| lowCards | 
	self setupGame.
	
	self assert: Chancellor cost = 3.
	
	"Play a Chancellor.  Make sure we gain $2 and the draw is cycled into the discard"
	deck1 z_addCardToHand: Chancellor.
	board execute: (ActionCommand deck: 1 card: Chancellor).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_draw size = 0.
	self assert: deck1 z_discard size = 5.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Chancellor.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 actions = 0.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testChapel
	| lowCards | 
	self setupGame.
	
	self assert: Chapel cost = 2.
	
	"Play a Cellar.  We are using a default player who will discard all of his victory cards, so make sure he does so and that the correct number of cards are drawn to replace them."
	lowCards := (deck1 hand select: [:each | (each class = Copper) or: (each class = Estate)]) size min: 4.
	deck1 z_addCardToHand: Chapel.
	board execute: (ActionCommand deck: 1 card: Chapel).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = (5 - lowCards).
	self assert: (board z_trash size = lowCards).
	self assert: (deck1 play at: 1) class = Chapel.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 actions = 0.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 16:30'!
testCouncilRoom
	self setupGame.
	
	self assert: CouncilRoom cost = 5.
	
	"Play a Council Room. Make sure 4 cards are drawn, 1 buy is added, and other players draw a card."

	deck1 z_addCardToHand: CouncilRoom.	

	board execute: (ActionCommand deck: 1 card: CouncilRoom).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 9.
	self assert: deck1 buys = 2.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = CouncilRoom.
	self assert: deck1 phase = #Treasure.
	self assert: deck2 hand size = 6.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:35'!
testEndTurnCommand
	self setupGame.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send an end turn command to deck1."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 1).
	self assert: player2 errorCommand isNil.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 5.
	self assert: deck1 z_draw size = 0.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 1).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an end turn command to deck2."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 5.
	self assert: deck2 z_draw size = 0.
	self assert: deck2 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send a play treasure command to deck1."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(4) play(1)"
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	
	"Send a buy command to deck1.  This also ends the turn because the deck is out of buys"
	"Before:     draw(0) discard(5) hand(4) play(1)"
	"After buy: draw(0) discard(6) hand(4) play(1)"
	"After end: draw(6) discard(0) hand(5) play(0)"
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 0.
	self assert: deck1 z_draw size = 6.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 1).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an end turn command to deck2."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:   draw(5) discard(0) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 0.
	self assert: deck2 z_draw size = 5.
	self assert: deck2 phase = #None.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testFeast
	self setupGame.
	
	self assert: Feast cost = 4.
	
	"Play a Feast.  The test player will choose to gain a silver.  Make sure he does so. Also ensure that the feast gets trashed."
	deck1 z_addCardToHand: Feast.
	board execute: (ActionCommand deck: 1 card: Feast).
	self assert: deck1 play size = 0.
	self assert: board trashTop class = Feast.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = 1).
	self assert: deck1 z_discard last class = Silver.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 18:17'!
testFestival
	self setupGame.
	
	self assert: Festival cost = 5.
	
	"Play a Festival. Make sure 2 actions, 1 buy, and 2 money are added"

	deck1 z_addCardToHand: Festival.	

	board execute: (ActionCommand deck: 1 card: Festival).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 actions = 2.
	self assert: deck1 buys = 2.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Festival.
	self assert: deck1 phase = #Action.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/29/2012 12:14'!
testGameEnd
	self setupGame.
	board  addCard: Gardens.
	
	"Test a game ending where one player has the most VPs"
	1 to: 7 do: [:a |
		4 timesRepeat: [deck1 z_addCardToHand: Gold].
		4 timesRepeat: [board execute: (TreasureCommand deck: 1 card: Gold)].
		
		deck1 addBuys: 1.
		board execute: (BuyCommand deck: 1 card: Province).
		board execute: (BuyCommand deck: 1 card: Gardens).
		
		board execute: (EndTurnCommand deck: 2).
		
		self assert: board gameOver not.
		self should: [deck1 victoryPoints] raise: DominionError.
	].
	
	1 to: 3 do: [:b | deck1 z_addCardToHand: Gold].
	1 to: 3 do: [:b |board execute: (TreasureCommand deck: 1 card: Gold)].
	board execute: (BuyCommand deck: 1 card: Province).
	
	self assert: board gameOver.
	self assert: (board winners at: 1) = 1.
	self assert: (board winners size) = 1.
	self assert: deck1 victoryPoints = ((6*8)+3+(7*(deck1 allCards size / 10) floor)).
	self assert: deck2 victoryPoints = 3.
	self assert: (player2 events count: [:each | each selector = #gameOverWinners:]) = 1.
	self assert: (player1 events count: [:each | each selector = #gameOverWinners:]) = 1.
	
	"Test a game ending where the VPs are tied but one player has taken less turns"
	self setupGame.
	
	1 to: 3 do: [:a |
		3 timesRepeat: [deck1 z_addCardToHand: Gold].
		3 timesRepeat: [board execute: (TreasureCommand deck: 1 card: Gold)].
		board execute: (BuyCommand deck: 1 card: Province).
		
		3 timesRepeat: [deck2 z_addCardToHand: Gold].
		3 timesRepeat: [board execute: (TreasureCommand deck: 2 card: Gold)].
		board execute: (BuyCommand deck: 2 card: Province).
		
		self assert: board gameOver not.
		self should: [deck1 victoryPoints] raise: DominionError.
	].
	
	board execute: (EndTurnCommand deck: 1).
	
	3 timesRepeat: [deck2 z_addCardToHand: Gold].
	3 timesRepeat: [board execute: (TreasureCommand deck: 2  card: Gold)].
	board execute: (BuyCommand deck: 2 card: Province).
		
	3 timesRepeat: [deck1 z_addCardToHand: Gold].
	3 timesRepeat: [board execute: (TreasureCommand deck: 1 card: Gold)].
	board execute: (BuyCommand deck: 1 card: Province).
	
	self assert: board gameOver.
	self assert: (board winners at: 1) = 2.
	self assert: (board winners size) = 1.
	self assert: deck1 victoryPoints = ((4*6) + 3).
	self assert: deck2 victoryPoints = ((4*6) + 3).
	self assert: (player2 events count: [:each | each selector = #gameOverWinners:]) = 1.
	self assert: (player1 events count: [:each | each selector = #gameOverWinners:]) = 1.
	
	
	"Test a game ending where the players are tied in both VPs and turns"
	self setupGame.
	1 to: 4 do: [:a |
		3 timesRepeat: [deck1 z_addCardToHand: Gold].
		3 timesRepeat: [board execute: (TreasureCommand deck: 1 card: Gold)].
		board execute: (BuyCommand deck: 1 card: Province).
		
		3 timesRepeat: [deck2 z_addCardToHand: Gold].
		3 timesRepeat: [board execute: (TreasureCommand deck: 2 card: Gold)].
		board execute: (BuyCommand deck: 2 card: Province).
	].
	
	self assert: board gameOver.
	self assert: (board winners size) = 2.
	self assert: deck1 victoryPoints = ((4*6) + 3).
	self assert: deck2 victoryPoints = ((4*6) + 3).
	self assert: (player2 events count: [:each | each selector = #gameOverWinners:]) = 1.
	self assert: (player1 events count: [:each | each selector = #gameOverWinners:]) = 1.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 16:34'!
testLaboratory
	self setupGame.
	
	self assert: Laboratory cost = 5.
	
	"Play a Laboratory. Make sure 2 cards are drawn and 1 action is added."

	deck1 z_addCardToHand: Laboratory.	

	board execute: (ActionCommand deck: 1 card: Laboratory).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 1) class = Laboratory.
	self assert: deck1 phase = #Action.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testLibrary
	self setupGame.
	
	self assert: Library cost = 5.
	
	"Play a Library.  Make sure that it draws cards until 7 in hand and discards any actions."
	deck1 z_draw addLast: Village basicNew.
	deck1 z_draw addLast: Copper basicNew.
	deck1 z_draw addLast: Village basicNew.
	deck1 z_addCardToHand: Library.
	board execute: (ActionCommand deck: 1 card: Library).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Library.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 resolving = false.
	self assert: deck1 z_discard size = 2.
	self assert: (deck1 z_discard at: 1) class = Village.
	self assert: (deck1 z_discard at: 2) class = Village.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/27/2012 20:39'!
testMarket
	self setupGame.
	
	self assert: Market cost = 5.
	
	"Play a Market. 1 card is draw, 1 action is added, 1 buy is added, and 1 money is added."

	deck1 z_addCardToHand: Market.	

	board execute: (ActionCommand deck: 1 card: Market).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 6.
	self assert: deck1 actions = 1.
	self assert: deck1 buys = 2.
	self assert: deck1 money = 1.
	self assert: (deck1 play at: 1) class = Market.
	self assert: deck1 phase = #Action.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 13:04'!
testMilitia
	self setupGame.
	
	self assert: Militia cost = 4.
	
	"Play a Bureaucrat.  Make sure that a silver is added to the top of the draw and that other players either placed a victory card on top of the draw or have no victory cards."
	deck1 z_addCardToHand: Militia.
	board execute: (ActionCommand deck: 1 card: Militia).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 money = 2.
	self assert: deck2 hand size = 3.
	self assert: deck2 z_discard size = 2.
	self assert: (deck1 play at: 1) class = Militia.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testMine
	| copper | 
	self setupGame.
	
	self assert: Mine cost = 5.
	
	"Play a Mine.  We are using a default player who will trash copper for silver, so make sure he does so."
	copper := (deck1 hand select: [:each | each class = Copper]) size.
	deck1 z_addCardToHand: Mine.
	board execute: (ActionCommand deck: 1 card: Mine).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: board trashTop class = Copper.
	self assert: (deck1 z_discard size = 0).
	self assert: (deck1 hand select: [:each | each class = Copper]) size = (copper - 1).
	self assert: (deck1 handContains: Silver).
	self assert: (deck1 play at: 1) class = Mine.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testMoat
	self setupGame.
	
	self assert: Moat cost = 2.
	
	"Play a Bureaucrat.  Make sure that nothing happens because player 2 reveals the moat."
	deck1 z_addCardToHand: Bureaucrat.
	deck2 z_addCardToHand: Moat.
	deck2 z_addCardToHand: Estate.
	board execute: (ActionCommand deck: 1 card: Bureaucrat).
	self assert: deck2 hand size = 7.
	self assert: deck1 resolving = false.
	
	self setupGame.
	
	"Play a Moat. Make sure 2 cards are drawn"
	deck1 z_addCardToHand: Moat.	

	board execute: (ActionCommand deck: 1 card: Moat).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moat.
	self assert: deck1 phase = #Treasure.

	
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 13:15'!
testMoneylender
	self setupGame.
	
	self assert: Moneylender cost = 4.
	
	"Play a Moneylender with copper in hand.  Make sure the copper is trashed and money is +3"

	deck1 z_addCardToHand: Moneylender.	

	board execute: (ActionCommand deck: 1 card: Moneylender).
	self assert: deck1 play size = 1.
	self assert: board trashTop class = Copper.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 3.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moneylender.
	self assert: deck1 phase = #Treasure.
	
	
	"Play a Moneylender with no copper in hand.  Make sure nothing happens."
	self setupGame.
	deck1 z_areas at: #hand put: CardArea new.
	deck1 z_addCardToHand: Moneylender.
	4 timesRepeat: [deck1 z_addCardToHand: Estate].	
	
	board execute: (ActionCommand deck: 1 card: Moneylender).
	self assert: deck1 play size = 1.
	self assert: board z_trash size = 0.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 0.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moneylender.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testRemodel
	|estates|
	self setupGame.
	
	self assert: Remodel cost = 4.
	
	"Play a Remodel.  We are using a default player who will trash an estate for silver, so make sure he does so."
	deck1 z_addCardToHand: Estate.
	estates := (deck1 hand select: [:each | each class = Estate]) size.
	deck1 z_addCardToHand: Remodel.
	board execute: (ActionCommand deck: 1 card: Remodel).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: board trashTop class = Estate.
	self assert: (deck1 z_discard size = 1).
	self assert: (deck1 z_discard last class = Silver).
	self assert: (deck1 hand select: [:each | each class = Estate]) size = (estates - 1).
	self assert: (deck1 play at: 1) class = Remodel.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/28/2012 19:37'!
testSetup
	| actions cards temp|
	self setupGame.

	self assert: (deck1 z_draw select: [:each | each class = Copper]) size + (deck1 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck1 z_draw select: [:each | each class = Estate]) size + (deck1 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: (deck2 z_draw select: [:each | each class = Copper]) size + (deck2 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck2 z_draw select: [:each | each class = Estate]) size + (deck2 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: ((board cardsRemaining: Copper) = (60 - (7*2))).
	self assert: ((board cardsRemaining: Estate) = 8).
	
	actions := {Adventurer. Village. Smithy. Cellar. Mine.}.
	cards := {Copper. Silver. Gold. Estate. Duchy. Province. Curse. Adventurer. Village. Smithy. Cellar. Mine.}.
	
	actions do: [:each | board addCard: each].
	
	temp := board cardsAvailable.
	self assert: (temp includesAll: cards).
	self assert: (cards includesAll: temp).
	
	temp := board actionsAvailable.
	self assert: (temp includesAll: actions).
	self assert: (actions includesAll: temp).
	actions do: [:each | (board cardsRemaining: each) = 10].
	
	self assert: (board cardStackEmpty: Village) not.
	10 timesRepeat: [board takeCard: Village deck: 1].
	self assert: (board cardStackEmpty: Village).
	
! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:04'!
testSmithy
	self setupGame.
	
	self assert: Smithy cost = 4.
	
	"Play a Smithy. Make sure 3 cards are drawn"

	deck1 z_addCardToHand: Smithy.	

	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testSpy
	self setupGame.
	
	self assert: Spy cost = 4.
	
	"Play a Spy.  Make sure that the player draws 2 cards and other players gain a curse."
	deck1 z_draw addLast: Estate basicNew.
	deck1 z_draw addLast: Estate basicNew.
	deck2 z_draw addLast: Copper basicNew.
	deck1 z_addCardToHand: Spy.
	board execute: (ActionCommand deck: 1 card: Spy).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 6.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 1) class = Spy.
	self assert: deck1 phase = #Action.
	self assert: deck1 resolving = false.
	self assert: deck1 z_draw size = 5.
	self assert: deck2 z_draw size = 5.
	self assert: deck1 z_discard last class = Estate.
	self assert: deck2 z_discard last class = Copper.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testThief
	self setupGame.
	
	self assert: Thief cost = 4.
	
	"Play a Spy.  Make sure that player 2 reveals the top 2 cards of his deck, and that player 1 takes 1 of them and the other is discarded."
	deck2 z_draw addLast: Copper basicNew.
	deck2 z_draw addLast: Estate basicNew.
	deck1 z_addCardToHand: Thief.
	board execute: (ActionCommand deck: 1 card: Thief).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Thief.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 resolving = false.
	self assert: deck1 z_draw size = 5.
	self assert: deck1 z_discard size = 1.
	self assert: deck2 z_draw size = 5.
	self assert: deck2 z_discard size = 1.
	self assert: deck2 z_discard last class = Estate.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testThroneRoom
	self setupGame.
	
	self assert: ThroneRoom cost = 4.
	
	"Play a Throne Room on a Village.  Make sure the village gets played twice."
	deck1 z_addCardToHand: ThroneRoom.
	deck1 z_addCardToHand: Village.
	board execute: (ActionCommand deck: 1 card: ThroneRoom).
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 4.
	self assert: deck1 phase = #Action.
	self assert: deck1 resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:37'!
testTreasureCommand
	self setupGame.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 1.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Send a treasure command to the wrong deck.  Make sure it returns an error."
	board execute: (TreasureCommand deck: 2 card: Copper).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 3.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Try to buy an estate.  This costs 2, which we have due to having played 2 coppers"
	board execute: (BuyCommand deck: 1 card: Estate).
	self assert: player1 errorCommand isNil.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Estate.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:05'!
testVillage
	self setupGame.
	
	self assert: Village cost = 3.
	
	"Play a Village. Make sure a card was drawn, the number of actions is now 2, and the phase is still #Action"
	deck1 z_addCardToHand: Village.
	deck1 z_addCardToHand: Smithy.
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Village).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 2.
	self assert: (deck1 play at: 1) class = Village.
	self assert: deck1 phase = #Action.
	
	"Make sure we can now play two more cards."
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 2) class = Smithy.
	self assert: deck1 phase = #Action.
	
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 3.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 3) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 20:26'!
testWitch
	self setupGame.
	
	self assert: Witch cost = 5.
	
	"Play a Witch.  Make sure that the player draws 2 cards and other players gain a curse."
	deck1 z_addCardToHand: Witch.
	board execute: (ActionCommand deck: 1 card: Witch).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 play at: 1) class = Witch.
	self assert: deck1 phase = #Treasure.
	self assert: (deck2 z_discard last class = Curse).
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:16'!
testWoodcutter
	self setupGame.
	
	self assert: Woodcutter cost = 3.
	
	"Play a Smithy. Make sure 3 cards are drawn"

	deck1 z_addCardToHand: Woodcutter.	

	board execute: (ActionCommand deck: 1 card: Woodcutter).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 actions = 0.
	self assert: deck1 buys = 2.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Woodcutter.
	self assert: deck1 phase = #Treasure.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 8/11/2012 02:26'!
testWorkshop
	self setupGame.
	
	self assert: Workshop cost = 3.
	
	"Play a Workshop.  The test player will choose to gain a silver.  Make sure he does so."
	deck1 z_addCardToHand: Workshop.
	board execute: (ActionCommand deck: 1 card: Workshop).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = 1).
	self assert: deck1 z_discard last class = Silver.
	self assert: (deck1 play at: 1) class = Workshop.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 resolving = false.
	
	! !

TestCase subclass: #DisplayTest
	instanceVariableNames: 'board deck1 deck2 player1 player2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!DisplayTest methodsFor: 'private' stamp: 'MichaelOSullivan 7/22/2012 14:24'!
setupGame
	board := Board new.
	board initialize.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/27/2012 23:27'!
test2PlayerGameTable
	self setupGame.
	board decks: 2.
	board  addCard: Adventurer.
	board  addCard: Bureaucrat.
	board  addCard: Cellar.
	board  addCard: Mine.
	board  addCard: Smithy.
	board  addCard: Village.
	board  addCard: Moat.
	board  addCard: Witch.
	board  addCard: Chancellor.
	board  addCard: Chapel.
	
	(GameTable new board: board deck: 1) openInWorld.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/27/2012 23:27'!
test3PlayerGameTable
	self setupGame.
	board decks: 3.
	board  addCard: Remodel.
	board  addCard: ThroneRoom.
	board  addCard: Woodcutter.
	board  addCard: Workshop.
	board  addCard: Smithy.
	board  addCard: Village.
	board  addCard: Moat.
	board  addCard: Spy.
	board  addCard: Feast.
	board  addCard: Festival.
	
	(GameTable new board: board deck: 1) openInWorld.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/27/2012 23:28'!
test4PlayerGameTable
	self setupGame.
	board decks: 4.
	board  addCard: Chancellor.
	board  addCard: Chapel.
	board  addCard: CouncilRoom.
	board  addCard: Festival.
	board  addCard: Gardens.
	board  addCard: Laboratory.
	board  addCard: Library.
	board  addCard: Militia.
	board  addCard: Moneylender.
	board  addCard: Spy.
	
	(GameTable new board: board deck: 1) openInWorld.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/29/2012 11:53'!
testKingdomCards
	| cards cardClasses table |
	self setupGame.
	board decks: 2.
	board  addCard: Adventurer.
	board  addCard: Bureaucrat.
	board  addCard: Cellar.
	board  addCard: Mine.
	board  addCard: Smithy.
	board  addCard: Village.
	board  addCard: Moat.
	board  addCard: Witch.
	board  addCard: Chancellor.
	board  addCard: Chapel.
	
	table := GameTable new board: board deck: 1.
	cards := table kingdom cards.
	self assert: cards size = 10.
	
	cardClasses := cards collect: [:each | each cardClass].
	self assert: (cardClasses includes: Adventurer).
	self assert: (cardClasses includes: Bureaucrat).
	self assert: (cardClasses includes: Cellar).
	self assert: (cardClasses includes: Mine).
	self assert: (cardClasses includes: Smithy).
	self assert: (cardClasses includes: Village).
	self assert: (cardClasses includes: Moat).
	self assert: (cardClasses includes: Witch).
	self assert: (cardClasses includes: Chancellor).
	self assert: (cardClasses includes: Chapel).! !

Object subclass: #Board
	instanceVariableNames: 'cards decks trash currentDeck gameOver globalEvents playerEvents playerObservers globalObservers'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Dominion'!

!Board commentStamp: 'JohnParker 7/30/2012 20:29' prior: 0!
A Board is a singleton representing the overall game state of Dominion.

Instance Variables
	cards:			Bag
	currentDeck:	Number
	decks:			OrderedCollection
	trash: 			OrderedCollection 
	gameOver:		Boolean
	globalEvents	Array
	playerEvents	Array

cards
	- a Bag containing the card classes available and the number left of each class

currentDeck
	- the deck whose turn it currently is.

decks
	- the list of decks in the game.  

trash
	- a list of cards that have been trashed so far.

gameOver 
	- indicates whether the game is over.

globalEvents
	- the events that are sent to all players when triggered.
	Signatures:
		deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2   "Deck with number = aNumber moved a card between two areas, one of which is globally visible"
		deck: aNumber took: aCardClass          "Deck with number = aNumber took a card of type aCardClass from the supply"
		deck: aNumber trashed: aCardClass     "Deck with number = aNumber trashed a card of type aCardClass"
		turnStarted: aNumber                           "Deck with number = aNumber started their turn"
		deckShuffled: aNumber	                        "Deck with number = aNumber shuffled their deck"
		deckTurnState: aNumber                      "Deck with number = aNumber changed a turn state variable.  E.g. actions, buys, or money"
		gameOverWinners: aCollection             "The game is over and the winners are the deck numbers in aCollection"
		
playerEvents
	- the events that are sent to a specific player when triggered.  These events carry information that is private to a given player.
	Signatures
		deck: aNumber cardMovedPlayer: aCardClass from: area1 to: area2 "Deck with number = aNumber moved a card between two areas, neither of which is globally visible"
		resolveAction: aCardClass message: aString  "Your deck has played an action of type aCardClass and it needs resolution described by aString.  Respond with the appropriate ResolveActionCommand."
		resolveAction: aCardClass deck: aNumber     "Your deck has played an attack that needs a resolution for each deck it attacked.  
														aNumber is the deck number of the deck you are currently resolving the attack for.  Respond with the appropriate ResolveActionCommand"
		attack: aCardClass							  "You are being attacked by a card of type aCardClass.  Reveal a Moat if you have one and then respond with the appropriate ResolveAttackCommand."
		command: aCommand error: aString		  "A command you issued was invalid.  aCommand is the command that went wrong.  aString describes your mistake in user suitable terms."!

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/28/2012 19:35'!
actionsAvailable
	^cards keys select: [:each | each inheritsFrom: ActionCard].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/28/2012 19:25'!
addCard: aCardClass
	cards at: aCardClass put: (self startingStackSize: aCardClass).! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/29/2012 20:01'!
addObserver: anObserver
	globalObservers add: anObserver.
	globalEvents do: [:event | self when: event send: event to: anObserver].
	! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/29/2012 20:01'!
addObserver: anObserver at: deckNum
	playerObservers add: anObserver.
	playerEvents do: [:event | (decks at: deckNum) when: event send: event to: anObserver].
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/28/2012 19:33'!
cardStackEmpty: aCardClass
	^(cards at: aCardClass) = 0! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/27/2012 18:24'!
cards: aCardSet
	aCardSet kingdomCards do: [:each | self addCard: each].! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/28/2012 19:35'!
cardsAvailable
	^cards keys! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/28/2012 19:32'!
cardsRemaining: aCardClass
	^cards at: aCardClass! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/29/2012 11:42'!
checkGameOver
	(cards at: Province) = 0 ifTrue: [gameOver := true.].
	
	(cards inject: 0 into: [:numEmpty :each | numEmpty + (each value = 0 ifTrue: [1] ifFalse: [0])]) >= 3 ifTrue: [gameOver := true.].
	
	gameOver ifTrue: [
		self triggerEvent: #gameOverWinners: withArguments: {self winners}.
	]! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:56'!
currentDeck
	^decks at: currentDeck! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:29'!
deck: aNumber
	^decks at: aNumber ifAbsent: [^nil]! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/28/2012 19:31'!
decks: aNumber
	| victoryNumber deck|
	cards := Dictionary new.
	
	"Add the standard cards to the game"
	cards at: Copper put: 60.
	cards at: Silver put: 40.
	cards at: Gold put: 30.
	
	victoryNumber := (aNumber = 2) ifTrue: [8] ifFalse: [12].
	cards at: Estate put: (victoryNumber + (3*aNumber)).
	cards at: Duchy put: victoryNumber.
	cards at: Province put: victoryNumber.
	cards at: Curse put: (10 * (aNumber - 1)).
	
	1 to: aNumber do: [:num | 
		deck := Deck number: num board: self.
		decks add: deck.
		globalEvents do: [:event | deck when: event send: event to: self].
	].
	currentDeck := 1.
	! !

!Board methodsFor: 'events' stamp: 'JohnParker 7/13/2012 13:30'!
doesNotUnderstand: aMessage
	(globalEvents includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	self triggerEvent: aMessage selector withArguments: aMessage arguments
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 21:28'!
events
	|list|
	list := OrderedCollection new.
	list addAll: globalEvents.
	list addAll: playerEvents.
	^list.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/30/2012 12:59'!
execute: aCommand
	gameOver ifTrue: [(self deck: aCommand deck) triggerEvent: #command:error: withArguments: {aCommand.  'The game is already over!!'}. ^self.].
	aCommand do: self.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
gameOver
	^gameOver.! !

!Board methodsFor: 'initialize-release' stamp: 'JohnParker 8/12/2012 15:36'!
initialize
	decks := OrderedCollection new.
	currentDeck := 1.
	cards := Dictionary new.
	trash := OrderedCollection new.
	gameOver := false.
	globalEvents := {#deck:cardMovedGlobal:from:to:. #turnStarted:. #deckShuffled. #deck:took:. #deck:trashed:. #deckTurnState:. #gameOverWinners:. #resolutionStep}.
	playerEvents := {#deck:cardMovedPlayer:from:to:. #resolveAction:message:. #resolveAction:deck:. #attack:. #command:error:.}.
	playerObservers := OrderedCollection new.
	globalObservers := OrderedCollection new.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/29/2012 11:32'!
nextDeck
	self checkGameOver.
	gameOver ifFalse: [
		currentDeck := (currentDeck rem: decks size) + 1.
		(decks at: currentDeck) startTurn.
	]! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:39'!
numDecks
	^decks size.! !

!Board methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/21/2012 08:36'!
numberOfPlayers
	^decks size! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/4/2012 23:43'!
otherDecks
	^((1 to: decks size) reject: [:num | num = currentDeck]) collect: [:num | decks at: num].! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 8/12/2012 15:12'!
resolving
	^decks contains: [:each | each resolving].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/4/2012 13:49'!
startGame
	(decks at: 1) startTurn.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/26/2012 12:57'!
startingStackSize: aCardClass
	^(aCardClass inheritsFrom: VictoryCard) ifTrue: [
		(decks size = 2) ifTrue: [8] ifFalse: [12]
	] ifFalse: [
		(aCardClass = Curse) ifTrue: [10*(decks size - 1)] ifFalse: [10].
	].! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/29/2012 11:27'!
takeCard: aCardClass deck: aNumber
	|cardNum|
	gameOver ifTrue: [DominionError new signal:  'The game is over!!'].
	cardNum := cards at: aCardClass ifAbsent: [DominionError new signal:  ('The current game does not contain the card' , aCardClass cardName)].
	cardNum > 0 ifFalse: [DominionError new signal:  (('There are no more ' , aCardClass cardName) , ' cards in the supply.')].
	
	cards at: aCardClass put: (cardNum - 1).
	self triggerEvent: #deck:took: withArguments: {aNumber. aCardClass}.
	^aCardClass basicNew.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/28/2012 19:59'!
takeTrashTop
	gameOver ifTrue: [DominionError new signal:  'The game is over!!'].
	^trash removeLast.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/28/2012 19:59'!
trashCard: aCard deck: deckNumber
	gameOver ifTrue: [DominionError new signal:  'The game is over!!'].
	trash addLast: aCard.
	self triggerEvent: #deck:trashed: withArguments: {deckNumber. aCard class}.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 12:06'!
trashCount
	^trash size! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 12:18'!
trashTop
	trash size = 0 ifTrue: [^nil].
	^trash last.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/29/2012 11:41'!
winners
	|tempDecks|
	gameOver ifFalse: [DominionError new signal: 'The game is not over so there is no winner yet.'].
	tempDecks := OrderedCollection new.
	tempDecks addAll: decks.
	
	"Sort based on Victory points and return the player with the highest VP if untied"
	tempDecks sort: [:first :second | first victoryPoints > second victoryPoints].
	tempDecks := tempDecks select: [:each | each victoryPoints = (tempDecks at: 1) victoryPoints].
	tempDecks size = 1 ifTrue: [^{(tempDecks at: 1) number}].
	
	"Sort the VP tied decks based on lowest number of turns and return the lowest turn deck if untied"
	tempDecks sort: [:first :second | first turns < second turns].
	tempDecks := tempDecks select: [:each | each turns = (tempDecks at: 1) turns].
	tempDecks size = 1 ifTrue: [^{(tempDecks at: 1) number}].
	
	"More than one deck has equal VP and turns.  Return them all"
	^tempDecks collect: [:each | each number].! !

!Board methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 11:56'!
z_trash
	^trash! !

Object subclass: #Card
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Card commentStamp: 'JohnParker 7/3/2012 19:31' prior: 0!
A Card is an abstract class at the top of the card hierarchy.  It represents a generic card in Dominion.!

Card subclass: #ActionCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCard commentStamp: 'JohnParker 7/4/2012 14:49' prior: 0!
An ActionCard is an abstract class representing a Dominion action card.  !

!ActionCard class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:27'!
color
	^Color white! !

!ActionCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 11:57'!
resolve: argVector board: board! !

ActionCard subclass: #Adventurer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Adventurer commentStamp: 'JohnParker 7/29/2012 23:41' prior: 0!
Reveal cards from your deck until you reveal 2 Treasure cards. Put those Treasure cards in your hand and discard the other revealed cards.!

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:01'!
cardName
	^'Adventurer'! !

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cost
	^6! !

!Adventurer methodsFor: 'game' stamp: 'JohnParker 7/30/2012 12:49'!
playCard: board
	| numTreasure card deck|
	numTreasure := 0.
	deck := board currentDeck.
	[numTreasure < 2] whileTrue: [
		card := deck drawCardTo: #reveal.
		card isNil ifTrue: [numTreasure := 2].
		
		(card class inheritsFrom: TreasureCard) ifTrue: [
			deck moveCard: card class from: #reveal to: #hand.
			numTreasure := numTreasure + 1
		] ifFalse: [
			deck moveCard: card class from: #reveal to: #discard.
		].
	].! !

ActionCard subclass: #AttackCard
	instanceVariableNames: 'playersResolved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!AttackCard commentStamp: 'JohnParker 7/29/2012 23:18' prior: 0!
An AttackCard is an abstract class representing a Dominion attack card.  

Instance Variables
	playersResolved:		OrderedCollection

playersResolved
	- The list of players that have resolved the attack upon themselves.
!

!AttackCard methodsFor: 'game' stamp: 'JohnParker 7/30/2012 00:10'!
cancelAttack: deckNumber board: board
	playersResolved at: deckNumber put: true.
	(board currentDeck) cardResolved: self resolved.
	! !

!AttackCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:58'!
resolveAttack: args board: board! !

!AttackCard methodsFor: 'game' stamp: 'JohnParker 7/30/2012 00:01'!
resolved
	^(playersResolved inject: true into: [:val :each | val and: each])! !

AttackCard subclass: #Bureaucrat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Bureaucrat commentStamp: 'JohnParker 7/29/2012 23:47' prior: 0!
Gain a silver card; put it on top of your deck. Each other player reveals a Victory card from his hand and puts it on his deck (or reveals a hand with no Victory cards).

Sends #attack: to all other players
ResolveAttackCommand arguments:
	#bureaucratCard - A VictoryCard subclass indicating which card to put back on top of the player's deck.  If nil, they must have no Victory cards.!

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:30'!
cardName
	^'Bureaucrat'! !

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 7/29/2012 23:58'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	
	board currentDeck gainCard: Silver to: #draw.
	board currentDeck cardResolved: false.
	board otherDecks do: [:each | each attack: self].! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 8/11/2012 00:26'!
resolveAttack: args board: board deck: deck
	|cardClass card|
	
	(playersResolved at: deck number) ifTrue: [DominionError new signal: 'That attack has already been resolved.'].
	
	cardClass := args at: #bureaucratCard.
	
	cardClass ifNil: [
		((deck hand select: [:each | each class inheritsFrom: VictoryCard]) size = 0) ifFalse: [
			DominionError new signal: 'You must choose a victory card if you have one.'
		].
		deck hand do: [:each | deck moveCard: each class from: #hand to: #reveal].
		deck reveal do: [:each | deck moveCard: each class from: #reveal to: #hand].
		playersResolved at: deck number put: true.
		board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
		deck attack: nil.
		^self.
	].
	
	(cardClass inheritsFrom: VictoryCard) ifFalse: [DominionError new signal: 'The card you choose must be a victory card.'].
	
	card := deck hand detect: [:each | each class = cardClass] ifNone: [DominionError new signal: 'You must choose a card you have in your hand.'].
	
	deck moveCard: card class from: #hand to: #reveal.
	deck moveCard: card class from: #reveal to: #draw.
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
	deck attack: nil.
	
	! !

AttackCard subclass: #Militia
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Militia commentStamp: 'JohnParker 7/29/2012 23:47' prior: 0!
+$2
Each other player discards down to 3 cards in his hand.

Sends #attack: to all other players
ResolveAttackCommand arguments:
	#discards - An Array of card classes to discard.!

!Militia class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:41'!
cardName
	^'Militia'! !

!Militia class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:41'!
cost
	^4! !

!Militia methodsFor: 'game' stamp: 'JohnParker 7/30/2012 00:01'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	board currentDeck cardResolved: false.
	
	board currentDeck addMoney: 2.
	board otherDecks do: [:each | each attack: self].! !

!Militia methodsFor: 'game' stamp: 'JohnParker 7/30/2012 00:03'!
resolveAttack: args board: board deck: deck
	|discards|
	
	(playersResolved at: deck number) ifTrue: [DominionError new signal: 'That attack has already been resolved.'].
	
	discards := args at: #discards.
	
	((deck hand size - discards size) = 3) ifFalse: [DominionError new signal: 'You must discard down to 3 cards in your hand.'].
	
	(deck handContainsAll: discards) ifFalse: [DominionError new signal: 'You can only discard the cards you have in your hand.'].
	
	discards do: [:each | deck moveCard: each from: #hand to: #discard].
	
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: self resolved.
	deck attack: nil.
	
	! !

AttackCard subclass: #Spy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Spy commentStamp: 'JohnParker 7/29/2012 23:52' prior: 0!
+1 Card; +1 Action
Each player (including you) reveals the top card of his deck and either discards it or puts it back, your choice.

Sends #resolveAction:deck: to the current player

Sends #attack: to all other players
ResolveAttackCommand arguments:
	None

When the ResolveAttackCommand comes back for a different player, it sends the #resolveAction:deck: to the current player
ResolveActionCommand arguments:
	#deck - the deck we are resolving this action for.
	#doDiscard - a boolean indicating whether the card just drawn should be discarded.

!

!Spy class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:27'!
cardName
	^'Spy'! !

!Spy class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:27'!
cost
	^4! !

!Spy methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:47'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	| deck|
	deck := board deck: aNumber.
	
	fromArea = #reveal ifFalse: [DominionError new signal: 'Spy can only move cards from a player''s reveal area.'].
	((toArea =  #discard) or: (toArea = #draw)) ifFalse: [DominionError new signal: 'Spy can only move discard the revealed card or put it back.'].
	aCardClass = deck reveal last class ifFalse: [DominionError new signal: 'Player ' , aNumber asString , ' has not revealed a ' , aCardClass cardName].
	
	toArea =  #discard ifTrue: [
		deck moveCard: deck reveal last class from: #reveal to: #discard
	] ifFalse: [
		deck moveCard: deck reveal last class from: #reveal to: #draw
	].
	
	playersResolved at: deck number put: true.
	deck attack: nil.
	
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: self resolved.! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:24'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].

	board currentDeck drawCardTo: #hand.
	board currentDeck addActions: 1.
	board currentDeck cardResolved: false.
	
	board otherDecks do: [:each | each attack: self].
	
	board currentDeck drawCardTo: #reveal.
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. board currentDeck number}.! !

!Spy methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:53'!
resolve: args board: board
	DominionError new signal: 'You must either trash or discard the cards revealed by the other players'.! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/27/2012 20:32'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [DominionError new signal: 'That attack has already been resolved.'].
	deck drawCardTo: #reveal.
	
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. deck number}.
	! !

AttackCard subclass: #Thief
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Thief commentStamp: 'JohnParker 7/29/2012 23:52' prior: 0!
Each other player reveals the top 2 cards of his deck. If they revealed any Treasure cards, they trash one of them that you choose. You may gain any or all of these trashed cards. They discard the other revealed cards.

Sends #attack: to all other players
ResolveAttackCommand arguments:
	None

When the ResolveAttackCommand comes back for a different player, it sends the #resolveAction:deck: to the current player
ResolveActionCommand arguments:
	#deck - the deck we are resolving this action for.
	#trash - the Card class to trash
	#take - a boolean indicating whether we want to take the card trashed!

!Thief class methodsFor: 'accessing' stamp: 'JohnParker 7/29/2012 22:00'!
cardName
	^'Thief'! !

!Thief class methodsFor: 'accessing' stamp: 'JohnParker 7/22/2012 21:11'!
cost
	^4! !

!Thief methodsFor: 'game' stamp: 'JohnParker 8/12/2012 21:26'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	| deck|
	deck := board deck: aNumber.
	
	fromArea = #reveal ifFalse: [DominionError new signal: 'Thief can only move cards from a player''s reveal area.'].
	((toArea =  #discard) or: (toArea = #trash)) ifFalse: [DominionError new signal: 'Thief can only trash the revealed card or gain it to your discard.'].
	
	(deck reveal contains: [:each | each class = aCardClass]) ifFalse: [DominionError new signal: 'Player ' , aNumber asString , ' has not revealed a ' , aCardClass cardName].
	
	deck trashCard: aCardClass from: #reveal.
	deck moveCard: deck reveal last class from: #reveal to: #discard.
	toArea =  #discard ifTrue: [
		board currentDeck takeTrashTopTo: #discard
	].

	playersResolved at: deck number put: true.
	deck attack: nil.
	
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: self resolved.! !

!Thief methodsFor: 'game' stamp: 'JohnParker 7/22/2012 21:49'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	
	board currentDeck cardResolved: false.
	
	board otherDecks do: [:each | each attack: self].! !

!Thief methodsFor: 'game' stamp: 'JohnParker 8/12/2012 21:13'!
resolve: args board: board
	DominionError new signal: 'You must either trash or gain the cards revealed by the other players'.! !

!Thief methodsFor: 'game' stamp: 'JohnParker 8/12/2012 21:22'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [DominionError new signal: 'That attack has already been resolved.'].
	
	2 timesRepeat: [deck drawCardTo: #reveal].
	
	(deck reveal contains: [:each | each class inheritsFrom: TreasureCard]) ifFalse: [
		2 timesRepeat: [deck moveCard: deck reveal last class from: #reveal to: #discard].
		playersResolved at: deck number put: true.
		deck attack: nil.
	
		board triggerEvent: #resolutionStep.
		board currentDeck cardResolved: self resolved.
		^self.
	] .
	
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. deck number}.
	! !

AttackCard subclass: #Witch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Witch commentStamp: 'JohnParker 7/29/2012 23:48' prior: 0!
+2 Cards
Each other player gains a Curse card.

Sends #attack: to all other players
ResolveAttackCommand arguments:
	None!

!Witch class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:20'!
cardName
	^'Witch'! !

!Witch class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:20'!
cost
	^5! !

!Witch methodsFor: 'game' stamp: 'JohnParker 7/29/2012 23:58'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	board currentDeck cardResolved: false.
	
	2 timesRepeat: [board currentDeck drawCardTo: #hand ].
	board otherDecks do: [:each | each attack: self].! !

!Witch methodsFor: 'game' stamp: 'JohnParker 7/31/2012 12:19'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [DominionError new signal: 'That attack has already been resolved.'].
	
	[deck gainCard: Curse to: #discard.] on: DominionError do: [].
	
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: self resolved.
	deck attack: nil.
	! !

ActionCard subclass: #Cellar
	instanceVariableNames: 'numberDiscarded'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Cellar commentStamp: 'JohnParker 7/29/2012 23:32' prior: 0!
+1 Action
Discard any number of cards.
+1 Card per card discarded.

Sends #resolveAction:
ResolveActionCommand arguments:
	#discards - An array of card classes to discard!

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cardName
	^'Cellar'! !

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^2! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:45'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	fromArea = #hand ifFalse: [DominionError new signal: 'Cellar only discards from your hand.'].
	toArea =  #discard ifFalse: [DominionError new signal: 'Cellar can only move cards to your discard pile.'].
	(deck handContains: aCardClass) ifFalse: [DominionError new signal: 'Cellar can only discard cards that you have in your hand.'].
	
	numberDiscarded := numberDiscarded + 1.
	deck moveCard: aCardClass from: #hand to: #discard.
	
	
	
	! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 8/11/2012 02:18'!
playCard: board
	board currentDeck addActions: 1.
	board currentDeck cardResolved: false.
	numberDiscarded := 0.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Discard any number of cards.  +1 Card per card discarded.'}.
	! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:43'!
resolve: args board: board
	| deck discards|
	deck := board currentDeck.
	
	numberDiscarded timesRepeat: [deck drawCardTo: #hand].
	board currentDeck cardResolved: true.
	deck triggerEvent: #resolutionStep.! !

ActionCard subclass: #Chancellor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Chancellor commentStamp: 'JohnParker 7/29/2012 23:34' prior: 0!
+$2
You may immediately put your deck into your discard pile.

Sends #resolveAction:
ResolveActionCommand arguments:
	#doDiscard - A boolean indicating whether the deck should be put into the discard.!

!Chancellor class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:06'!
cardName
	^'Chancellor'! !

!Chancellor class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:06'!
cost
	^3! !

!Chancellor methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:46'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	fromArea = #draw ifFalse: [DominionError new signal: 'Chancellor can only move your deck into your discard pile.'].
	toArea =  #discard ifFalse: [DominionError new signal: 'Chancellor can only move your deck into your discard pile.'].
	
	deck drawSize timesRepeat: [deck drawCardTo: #discard].
	
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true. ! !

!Chancellor methodsFor: 'game' stamp: 'JohnParker 7/30/2012 20:25'!
playCard: board
	board currentDeck addMoney: 2.
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'You may immediately put your deck into your discard pile.'}.
	! !

!Chancellor methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:46'!
resolve: args board: board
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true. ! !

ActionCard subclass: #Chapel
	instanceVariableNames: 'numberTrashed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Chapel commentStamp: 'JohnParker 7/29/2012 23:32' prior: 0!
Trash up to 4 cards from your hand.

Sends #resolveAction:
ResolveActionCommand arguments:
	#trash - An array of card classes to trash!

!Chapel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 11:47'!
cardName
	^'Chapel'! !

!Chapel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 11:46'!
cost
	^2! !

!Chapel methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:46'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	fromArea = #hand ifFalse: [DominionError new signal: 'Chapel only trashes from your hand.'].
	toArea =  #trash ifFalse: [DominionError new signal: 'Chapel can only trash cards.'].
	(deck handContains: aCardClass) ifFalse: [DominionError new signal: 'Chapel can only trash cards that you have in your hand.'].
	(numberTrashed < 4) ifFalse: [DominionError new signal: 'Chapel can only trash up to 4 cards.'].
	
	numberTrashed := numberTrashed + 1.
	deck trashCard: aCardClass from: #hand.! !

!Chapel methodsFor: 'game' stamp: 'JohnParker 8/11/2012 12:30'!
playCard: board
	board currentDeck cardResolved: false.
	numberTrashed := 0.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Trash up to 4 cards.'}.
	! !

!Chapel methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:46'!
resolve: args board: board
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true.! !

ActionCard subclass: #CouncilRoom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!CouncilRoom commentStamp: 'JohnParker 7/29/2012 23:40' prior: 0!
+4 Cards; +1 Buy
Each other player draws a card.!

!CouncilRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/28/2012 16:36'!
cardName
	^'Council Room'! !

!CouncilRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:28'!
cost
	^5! !

!CouncilRoom methodsFor: 'game' stamp: 'JohnParker 7/20/2012 16:30'!
playCard: board
	board currentDeck addBuys: 1.
	4 timesRepeat: [board currentDeck drawCardTo: #hand].
	board otherDecks do: [:each | each drawCardTo: #hand].! !

ActionCard subclass: #Feast
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Feast commentStamp: 'JohnParker 7/29/2012 23:39' prior: 0!
Trash this card. Gain a card costing up to $5.

Sends #resolveAction:
ResolveActionCommand arguments:
	#gainCard - a Card class indicating which card to gain
!

!Feast class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cardName
	^'Feast'! !

!Feast class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cost
	^4! !

!Feast methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:46'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	fromArea = #board ifFalse: [DominionError new signal: 'Feast only gains cards from the supply.'].
	toArea =  #discard ifFalse: [DominionError new signal: 'Feast only gains cards to your discard.'].
	((board cardsRemaining: aCardClass) > 0) ifFalse: [DominionError new signal: (('There are no more ' , aCardClass cardName) , ' cards in the supply.')].
	aCardClass cost <= 5 ifFalse: [DominionError new signal: 'Feast can only gain a card costing up to 5.']. 
	
	deck trashCard: self class from: #play.
	deck gainCard: aCardClass to: #discard.
	
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true.! !

!Feast methodsFor: 'game' stamp: 'JohnParker 7/30/2012 20:22'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Gain a card costing up to 5.'}.
	! !

!Feast methodsFor: 'game' stamp: 'JohnParker 8/11/2012 02:37'!
resolve: args board: board
	DominionError new signal: 'You must select a card costing up to 5 to gain.'.! !

ActionCard subclass: #Festival
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Festival commentStamp: 'JohnParker 7/29/2012 23:40' prior: 0!
+2 Actions, +1 Buy; +$2.!

!Festival class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:35'!
cardName
	^'Fesitval'! !

!Festival class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:35'!
cost
	^5! !

!Festival methodsFor: 'game' stamp: 'JohnParker 7/20/2012 19:25'!
playCard: board
	((board currentDeck addActions: 2) addBuys: 1) addMoney: 2.! !

ActionCard subclass: #Laboratory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Laboratory commentStamp: 'JohnParker 7/29/2012 23:40' prior: 0!
+2 Cards; +1 Action.!

!Laboratory class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:32'!
cardName
	^'Laboratory'! !

!Laboratory class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:32'!
cost
	^5! !

!Laboratory methodsFor: 'game' stamp: 'JohnParker 7/20/2012 16:32'!
playCard: board
	board currentDeck addActions: 1.
	2 timesRepeat: [board currentDeck drawCardTo: #hand].! !

ActionCard subclass: #Library
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Library commentStamp: 'JohnParker 7/29/2012 23:37' prior: 0!
Draw until you have 7 cards in hand. You may set aside any Action cards drawn this way, as you draw them; discard the set aside cards after you finish drawing.

Sends #resolveAction:
ResolveActionCommand arguments:
	#doDiscard - a boolean indicating whether the action card just drawn should be discarded.!

!Library class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:46'!
cardName
	^'Library'! !

!Library class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:46'!
cost
	^5! !

!Library methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:46'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	| deck|
	deck := board currentDeck.
	
	fromArea = #hand ifFalse: [DominionError new signal: 'Library can only set cards aside from your hand.'].
	toArea =  #reveal ifFalse: [DominionError new signal: 'Library can only set cards aside to your reveal area.'].
	(aCardClass inheritsFrom: ActionCard) ifFalse: [DominionError new signal: 'Library can only set aside action cards'].
	aCardClass = deck hand last class ifFalse: [DominionError new signal: 'Library can only set aside the most recently drawn card.'].
	
	deck moveCard: deck hand last class from: #hand to: #reveal.
	
	self resolve: nil board: board.! !

!Library methodsFor: 'game' stamp: 'JohnParker 8/11/2012 22:48'!
playCard: board
	|args|
	board currentDeck cardResolved: false.
	
	self resolve: nil board: board.
	! !

!Library methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:46'!
resolve: args board: board
	| deck discard lastDrawn|
	deck := board currentDeck.
	lastDrawn := nil.
	[(deck hand size < 7) and: (lastDrawn class inheritsFrom: ActionCard) not] whileTrue: [
		deck drawCardTo: #hand.
		lastDrawn := deck hand last.
	].	
	
	(lastDrawn class inheritsFrom: ActionCard) ifTrue: [
		board currentDeck cardResolved: false.
		board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'You may set aside the action card just drawn.'}.
		^self.
	].
	
	deck reveal size timesRepeat: [deck moveCard: deck reveal last class from: #reveal to: #discard.].
	
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Market
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Market commentStamp: 'JohnParker 7/29/2012 23:41' prior: 0!
+1 Card; +1 Action; +1 Buy; +$1.!

!Market class methodsFor: 'accessing' stamp: 'JohnParker 7/27/2012 20:37'!
cardName
	^'Market'! !

!Market class methodsFor: 'accessing' stamp: 'JohnParker 7/27/2012 20:38'!
cost
	^5! !

!Market methodsFor: 'game' stamp: 'JohnParker 7/27/2012 18:17'!
playCard: board
	|deck|
	deck := board currentDeck.
	deck drawCardTo: #hand.
	deck addActions: 1.
	deck addBuys: 1.
	deck addMoney: 1.
	! !

ActionCard subclass: #Mine
	instanceVariableNames: 'trashedCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Mine commentStamp: 'JohnParker 7/29/2012 23:39' prior: 0!
Trash a Treasure card from your hand. Gain a Treasure card costing up to $3 more; put it into your hand.

Sends #resolveAction:
ResolveActionCommand arguments:
	#trashCard - a Card class indicating which card to trash
	#gainCard - a Card class indicating which card to gain!

!Mine class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:50'!
cardName
	^'Mine'! !

!Mine class methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:48'!
cost
	^5! !

!Mine methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:46'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	(aCardClass inheritsFrom: TreasureCard) ifFalse: [DominionError new signal: 'Mine can only be used on Treasure Cards.'].
	
	fromArea = #hand ifTrue: [
		toArea = #trash ifFalse: [DominionError new signal: 'Mine only trashes cards from your hand.'].
		(deck handContains: aCardClass) ifFalse: [DominionError new signal: 'You do not have a ' , aCardClass cardName , 'in your hand to trash.'].
		
		trashedCost := aCardClass cost.
		deck trashCard: aCardClass from: #hand.
		^self.
	].
	
	fromArea = #board ifTrue: [
		toArea = #hand ifFalse: [DominionError new signal: 'Mine only gains cards to your hand.'].
		trashedCost ifNil: [DominionError new signal: 'You must trash a Treasure Card before gaining one.'].
		(board cardsRemaining: aCardClass) > 0 ifFalse: [DominionError new signal: 'There are no more ' , aCardClass cardName , ' cards in the supply.'].
		aCardClass cost > (trashedCost +3) ifTrue: [DominionError new signal: 'You can only gain a treasure costing at most 3 more than the trashed card.'].
		
		deck gainCard: aCardClass to: #hand.
		board currentDeck cardResolved: true.
		^self.
	].
	
	board triggerEvent: #resolutionStep.
	DominionError new signal: 'Mine only trashes cards in hand or gains cards to your hand.'
	! !

!Mine methodsFor: 'game' stamp: 'JohnParker 8/12/2012 13:18'!
playCard: board
	board currentDeck cardResolved: false.
	trashedCost := nil.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Trash a treasure from your hand.  Gain a treasure costing up to 3 more.'}.
	! !

!Mine methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:28'!
resolve: args board: board
	DominionError new signal: 'You must trash a card and gain a card'.! !

ActionCard subclass: #Moneylender
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Moneylender commentStamp: 'JohnParker 7/29/2012 23:40' prior: 0!
Trash a Copper from your hand. If you do, +$3.!

!Moneylender class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 13:06'!
cardName
	^'Moneylender'! !

!Moneylender class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 13:06'!
cost
	^4! !

!Moneylender methodsFor: 'game' stamp: 'JohnParker 7/20/2012 13:08'!
playCard: board
	|deck|
	deck := board currentDeck.
	(deck trashCard: Copper from: #hand) ifTrue: [
		deck addMoney: 3.
	].
	
	! !

ActionCard subclass: #Remodel
	instanceVariableNames: 'trashedCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Remodel commentStamp: 'JohnParker 7/29/2012 23:39' prior: 0!
Trash a card from your hand. Gain a card costing up to $2 more than the trashed card.

Sends #resolveAction:
ResolveActionCommand arguments:
	#trashCard - a Card class indicating which card to trash
	#gainCard - a Card class indicating which card to gain!

!Remodel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:14'!
cardName
	^'Remodel'! !

!Remodel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:14'!
cost
	^4! !

!Remodel methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:47'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	fromArea = #hand ifTrue: [
		toArea = #trash ifFalse: [DominionError new signal: 'Remodel only trashes cards from your hand.'].
		(deck handContains: aCardClass) ifFalse: [DominionError new signal: 'You do not have a ' , aCardClass cardName , 'in your hand to trash.'].
		
		trashedCost := aCardClass cost.
		deck trashCard: aCardClass from: #hand.
		^self.
	].
	
	fromArea = #board ifTrue: [
		toArea = #discard ifFalse: [DominionError new signal: 'Remodel only gains cards to your discard pile.'].
		trashedCost ifNil: [DominionError new signal: 'You must trash a card before gaining one.'].
		(board cardsRemaining: aCardClass) > 0 ifFalse: [DominionError new signal: 'There are no more ' , aCardClass cardName , ' cards in the supply.'].
		aCardClass cost > (trashedCost +2) ifTrue: [DominionError new signal: 'You can only gain a card costing at most 2 more than the trashed card.'].
		
		deck gainCard: aCardClass to: #discard.
		board triggerEvent: #resolutionStep.
		board currentDeck cardResolved: true.
		^self.
	].

	DominionError new signal: 'Remodel only trashes cards in hand or gains cards to your discard.'
	! !

!Remodel methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:25'!
playCard: board
	board currentDeck cardResolved: false.
	trashedCost := nil.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Trash a card from your hand.  Gain a card costing up to 2 more.'}.
	! !

!Remodel methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:28'!
resolve: args board: board
	DominionError new signal: 'You must trash a card and gain a card'.! !

ActionCard subclass: #ResponseCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResponseCard commentStamp: 'JohnParker 7/29/2012 23:27' prior: 0!
A ResponseCard is a card that can be revealed in response to an attack. 
!

ResponseCard subclass: #Moat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Moat commentStamp: 'JohnParker 7/29/2012 23:33' prior: 0!
+2 Cards
When another player plays an Attack card, you may reveal this from your hand. If you do, you are unaffected by that Attack.

May be revealed with a RevealResponseCommand.!

!Moat class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:01'!
cardName
	^'Moat'! !

!Moat class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:01'!
cost
	^2! !

!Moat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	|deck|
	deck := board currentDeck.
	2 timesRepeat: [deck drawCardTo: #hand].
	! !

!Moat methodsFor: 'game' stamp: 'JohnParker 7/30/2012 00:11'!
revealResponse: board deck: deck
	deck attack: nil.! !

!ResponseCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 19:57'!
revealResponse: board deck: deck! !

ActionCard subclass: #Smithy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Smithy commentStamp: 'JohnParker 7/29/2012 23:40' prior: 0!
+3 Cards.!

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:57'!
cardName
	^'Smithy'! !

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Smithy methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	3 timesRepeat: [board currentDeck drawCardTo: #hand].! !

ActionCard subclass: #ThroneRoom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!ThroneRoom commentStamp: 'JohnParker 7/29/2012 23:36' prior: 0!
Choose an Action card in your hand. Play it twice.

Sends #resolveAction:
ResolveActionCommand arguments:
	#playCard - a Card class indicating which card to play twice!

!ThroneRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:28'!
cardName
	^'Throne Room'! !

!ThroneRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:28'!
cost
	^4! !

!ThroneRoom methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:47'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck card|
	deck := board currentDeck.
	
	fromArea = #hand ifFalse: [DominionError new signal: 'Throne Room only plays cards from your hand.'].
	toArea =  #play ifFalse: [DominionError new signal: 'Throne Room only plays cards.'].
	(deck handContains: aCardClass) ifFalse: [DominionError new signal: 'You do not have a ' , aCardClass cardName , 'in your hand.'.].
	(aCardClass inheritsFrom: ActionCard) ifFalse: [DominionError new signal: 'Feast can only play action cards.']. 
	
	card := deck hand detect: [:each | each class = aCardClass].
	card playCard: board.
	card play: board.
	
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true.! !

!ThroneRoom methodsFor: 'game' stamp: 'JohnParker 7/30/2012 20:28'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Choose an action card to play twice.'}.
	! !

!ThroneRoom methodsFor: 'game' stamp: 'JohnParker 8/11/2012 02:58'!
resolve: args board: board
	DominionError new signal: 'You must select a card to play twice.'.! !

ActionCard subclass: #Village
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Village commentStamp: 'JohnParker 7/29/2012 23:37' prior: 0!
+1 Card; +2 Actions.!

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
cardName
	^'Village'! !

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^3! !

!Village methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	|deck|
	deck := board currentDeck.
	deck drawCardTo: #hand.
	deck addActions: 2.
	! !

ActionCard subclass: #Woodcutter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Woodcutter commentStamp: 'JohnParker 7/29/2012 23:38' prior: 0!
+1 Buy; +$2.!

!Woodcutter class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:13'!
cardName
	^'Woodcutter'! !

!Woodcutter class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:13'!
cost
	^3! !

!Woodcutter methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:14'!
playCard: board
	|deck|
	deck := board currentDeck.
	deck addMoney: 2.
	deck addBuys: 1.
	! !

ActionCard subclass: #Workshop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Workshop commentStamp: 'JohnParker 7/29/2012 23:38' prior: 0!
Gain a card costing up to $4.

Sends #resolveAction:
ResolveActionCommand arguments:
	#gainCard - a Card class indicating which card to gain!

!Workshop class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cardName
	^'Workshop'! !

!Workshop class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cost
	^3! !

!Workshop methodsFor: 'game' stamp: 'JohnParker 8/12/2012 15:47'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board
	|deck|
	deck := board currentDeck.
	
	fromArea = #board ifFalse: [DominionError new signal: 'Workshop only gains cards from the supply.'].
	toArea =  #discard ifFalse: [DominionError new signal: 'Workshop only gains cards to your discard.'].
	((board cardsRemaining: aCardClass) > 0) ifFalse: [DominionError new signal: (('There are no more ' , aCardClass cardName) , ' cards in the supply.')].
	aCardClass cost <= 4 ifFalse: [DominionError new signal: 'Workshop can only gain a card costing up to 4.']. 
	
	deck gainCard: aCardClass to: #discard.
	board triggerEvent: #resolutionStep.
	board currentDeck cardResolved: true.! !

!Workshop methodsFor: 'game' stamp: 'JohnParker 7/30/2012 20:28'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction:message: withArguments: {self class. 'Gain a card costing up to 4.'}.
	! !

!Workshop methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:27'!
resolve: args board: board
	DominionError new signal: 'You must select a card costing up to 4 to gain.'.! !

!Card class methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:08'!
buy: board
	^board currentDeck gainCard: self to: #discard.! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^'Card' ! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 00:19'!
color
	^Color cyan! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 8/10/2012 00:19'!
cost
	^nil.! !

!Card class methodsFor: 'instance creation' stamp: 'JohnParker 7/27/2012 20:29'!
new
	DominionError new signal: 'Cards can only be created by the Board'! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^self class cardName.! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^self class cost.! !

!Card methodsFor: 'game' stamp: 'JohnParker 7/27/2012 20:29'!
play: board
	|ret|
	(board currentDeck moveCard:  self class from: #hand to: #play) ifFalse: [DominionError new signal: 'You do not have a ' , self cardName , ' in your hand!!'].
	
	[ret := self playCard: board] on: DominionError do: [:e | 
		board currentDeck moveCard: self class from: #play to: #hand.
		e pass.
	].
	^ret.! !

!Card methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:47'!
playCard: board
	self SubclassResponsibility ! !

Card subclass: #CurseCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

CurseCard subclass: #Curse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Curse commentStamp: 'JohnParker 7/29/2012 23:44' prior: 0!
A Curse is a VictoryCard worth -1.
!

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cardName 
	^'Curse'! !

!Curse class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 22:40'!
color
	^Color magenta! !

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cost
	^0! !

!Curse methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:29'!
value: board deck: deck
	^-1! !

!CurseCard class methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 10:31'!
cardName 
	^'CurseCard'! !

!CurseCard class methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 10:32'!
color
	^Color magenta! !

!CurseCard methodsFor: 'private' stamp: 'JohnParker 8/11/2012 10:31'!
playCard: board
	DominionError new signal: 'Curse cards can not be played'! !

!CurseCard methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 10:31'!
value
	self SubclassResponsibility ! !

Card subclass: #TreasureCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCard commentStamp: 'JohnParker 7/4/2012 15:05' prior: 0!
A TreasureCard is an abstract class representing a Dominion treasure card.  It adds the value message which returns the amount of treasure the card is worth.!

TreasureCard subclass: #Copper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Copper commentStamp: 'JohnParker 7/29/2012 23:41' prior: 0!
A Copper is a Treasure worth 1!

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Copper'! !

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:45'!
cost
	^0! !

!Copper methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:22'!
value
	^1! !

TreasureCard subclass: #Gold
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Gold commentStamp: 'JohnParker 7/29/2012 23:42' prior: 0!
A Silver is a Treasure worth 3!

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Gold'! !

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^6! !

!Gold methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
value
	^3! !

TreasureCard subclass: #Silver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Silver commentStamp: 'JohnParker 7/29/2012 23:42' prior: 0!
A Silver is a Treasure worth 2!

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Silver'! !

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^3! !

!Silver methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:58'!
value
	^2! !

!TreasureCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:40'!
cardName
	^'Treasure'! !

!TreasureCard class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:28'!
color
	^Color yellow! !

!TreasureCard methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:58'!
playCard: board
	board currentDeck addMoney: self value.! !

!TreasureCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility! !

Card subclass: #VictoryCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!VictoryCard commentStamp: 'JohnParker 7/4/2012 15:06' prior: 0!
A VictoryCard is an abstract class representing a Dominion victory card.  It adds the value message which returns the number of victory points the card is worth. 
It also overrides the playCard message to generate an error since victory cards can not be played.!

VictoryCard subclass: #Duchy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Duchy commentStamp: 'JohnParker 7/29/2012 23:42' prior: 0!
A Duchy is a VictoryCard worth 3!

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cardName 
	^'Duchy'! !

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^5! !

!Duchy methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:29'!
value: board deck: deck
	^3! !

VictoryCard subclass: #Estate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Estate commentStamp: 'JohnParker 7/29/2012 23:42' prior: 0!
An Estate is a VictoryCard worth 1!

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cardName 
	^'Estate'! !

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cost
	^2! !

!Estate methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:30'!
value: board deck: deck
	^1! !

VictoryCard subclass: #Gardens
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Gardens commentStamp: 'JohnParker 7/29/2012 23:40' prior: 0!
Worth 1 Victory for every 10 cards in your deck (rounded down).!

!Gardens class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:28'!
cardName 
	^'Gardens'! !

!Gardens class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:28'!
cost
	^4! !

!Gardens methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:32'!
value: board deck: deck
	^(deck allCards size / 10) floor.! !

VictoryCard subclass: #Province
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Province commentStamp: 'JohnParker 7/29/2012 23:43' prior: 0!
A Province is a VictoryCard worth 6.  The game ends if they are all bought.!

!Province class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/15/2012 17:18'!
cardName 
	^'Province'! !

!Province class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^8! !

!Province methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:30'!
value: board deck: deck
	^6! !

!VictoryCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:47'!
cardName 
	^'Victory'! !

!VictoryCard class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:28'!
color
	^Color green! !

!VictoryCard methodsFor: 'private' stamp: 'JohnParker 7/27/2012 20:30'!
playCard: board
	DominionError new signal: 'Victory cards can not be played'! !

!VictoryCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility ! !

Object subclass: #CardMemento
	instanceVariableNames: 'position owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!CardMemento commentStamp: 'MichaelOSullivan 7/30/2012 17:34' prior: 0!
CardMemento objects remember the position and original owner of CardMorph (or any Morph) so that Morph can be restored to that state at a later time.
!

!CardMemento methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/24/2012 21:41'!
owner
	^ owner! !

!CardMemento methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/24/2012 21:41'!
owner: anObject
	owner := anObject! !

!CardMemento methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/24/2012 21:41'!
position
	^ position! !

!CardMemento methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/24/2012 21:41'!
position: aPoint
	position := aPoint! !

Object subclass: #CardSet
	instanceVariableNames: 'kingdomCards standardCards'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardSet commentStamp: 'JohnParker 7/29/2012 23:22' prior: 0!
A CardSet is a set of cards to be used to initialize a Dominion game.  It also has a variety of class methods for building card sets in different ways.

Instance Variables
	kingdomCards:	OrderedCollection
	standardCards:	 OrderedCollection

kingdomCards
	- The set of kingdom cards that will be available.

standardCards
	- The set of standard cards that will be available.  Only changes in the Prosperity expansion.
!

!CardSet class methodsFor: 'accessing' stamp: 'JohnParker 7/27/2012 18:17'!
basicSet
	^{Cellar.
		Chapel.
		Moat.
		Chancellor.
		Village.
		Woodcutter.
		Workshop.
		Bureaucrat.
		Feast.
		Gardens.
		Militia.
		Moneylender.
		Remodel.
		Smithy.
		Spy.
		Thief.
		ThroneRoom.
		CouncilRoom.
		Festival.
		Laboratory.
		Library.
		Market.
		Mine.
		Witch.
		Adventurer}! !

!CardSet class methodsFor: 'accessing' stamp: 'JohnParker 7/27/2012 18:17'!
standardCards
	^{Copper.
		Silver.
		Gold.
		Estate.
		Duchy.
		Province .
		Curse}! !

!CardSet methodsFor: 'initialize-release' stamp: 'JohnParker 7/27/2012 21:04'!
basicBigMoney
	kingdomCards := {Adventurer.
						Bureaucrat.
						Chancellor.
						Chapel.
						Feast.
						Laboratory.
						Market.
						Mine.
						Moneylender.
						ThroneRoom}.
	standardCards := self class standardCards.! !

!CardSet methodsFor: 'initialize-release' stamp: 'JohnParker 7/27/2012 21:06'!
basicFirstGame
	kingdomCards := {Cellar.
						Market.
						Militia.
						Mine.
						Moat.
						Remodel.
						Smithy.
						Village.
						Woodcutter.
						Workshop}.
	standardCards := self class standardCards.! !

!CardSet methodsFor: 'initialize-release' stamp: 'JohnParker 7/27/2012 21:07'!
basicInteraction
	kingdomCards := {Bureaucrat.
						Chancellor.
						CouncilRoom.
						Festival.
						Library.
						Militia.
						Moat.
						Spy.
						Thief.
						Village}.
	standardCards := self class standardCards.! !

!CardSet methodsFor: 'initialize-release' stamp: 'JohnParker 7/27/2012 18:27'!
basicRandom
	|allCards|
	allCards := self class basicSet.
	allCards shuffle.
	kingdomCards := allCards first: 10.
	standardCards := self class standardCards.! !

!CardSet methodsFor: 'initialize-release' stamp: 'JohnParker 7/27/2012 21:08'!
basicSizeDistortion
	kingdomCards := {Cellar.
						Chapel.
						Feast.
						Gardens.
						Laboratory.
						Thief.
						Village.
						Witch.
						Woodcutter.
						Workshop}.
	standardCards := self class standardCards.! !

!CardSet methodsFor: 'initialize-release' stamp: 'JohnParker 7/27/2012 21:09'!
basicVillageSquare
	kingdomCards := {Bureaucrat.
						Cellar.
						Festival.
						Library.
						Market.
						Remodel.
						Smithy.
						ThroneRoom.
						Village.
						Woodcutter}.
	standardCards := self class standardCards.! !

!CardSet methodsFor: 'accessing' stamp: 'JohnParker 7/26/2012 13:05'!
kingdomCards
	^kingdomCards.! !

!CardSet methodsFor: 'accessing' stamp: 'JohnParker 7/26/2012 13:05'!
standardCards
	^standardCards.! !

Object subclass: #Command
	instanceVariableNames: 'deckNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Command commentStamp: 'JohnParker 7/4/2012 14:53' prior: 0!
A Command is an abstract class implementing the command object pattern.  Its reciever is specified by deckNumber.

Instance Variables
	deckNumber:		Number

deckNumber
	- The deck number of the reciever.
!

Command subclass: #CardCommand
	instanceVariableNames: 'cardClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardCommand commentStamp: 'JohnParker 7/4/2012 14:52' prior: 0!
A CardCommand is an abstract class representing a Command that accepts a card class as a parameter

Instance Variables
	cardClass:		Card class

cardClass
	- passed to the reciever as an argument
!

CardCommand subclass: #ActionCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCommand commentStamp: 'JohnParker 7/4/2012 14:50' prior: 0!
An ActionCommand is a command object that sends the playAction: cardClass message to the deck specified by deckNumber!

!ActionCommand methodsFor: 'run' stamp: 'JohnParker 7/27/2012 20:34'!
do: board
	[(board deck: deckNumber) playAction: cardClass] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

CardCommand subclass: #BuyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!BuyCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A BuyCommand is a command object that sends the buy: cardClass message to the deck specified by deckNumber!

!BuyCommand methodsFor: 'run' stamp: 'JohnParker 7/27/2012 20:34'!
do: board
	[(board deck: deckNumber) buy: cardClass] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

!CardCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	^self new deck: aNumber card: aCardClass.! !

!CardCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	deckNumber := aNumber.
	cardClass := aCardClass.
	! !

CardCommand subclass: #MoveCardResolveCommand
	instanceVariableNames: 'fromArea toArea'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!MoveCardResolveCommand class methodsFor: 'instance creation' stamp: 'JohnParker 8/11/2012 02:21'!
deck: aNumber card: aCardClass from: area1 to: area2
	^self new deck: aNumber card: aCardClass from: area1 to: area2.! !

!MoveCardResolveCommand methodsFor: 'initialize-release' stamp: 'JohnParker 8/12/2012 14:49'!
deck: aNumber card: aCardClass from: area1 to: area2
	self deck: aNumber card: aCardClass.
	fromArea := area1.
	toArea := area2.
	deckNumber := aNumber! !

!MoveCardResolveCommand methodsFor: 'run' stamp: 'JohnParker 8/12/2012 14:50'!
do: board
	[(board currentDeck) moveCardResolve: cardClass from: fromArea to: toArea deck: deckNumber] on: DominionError do: [:e | 
		(board currentDeck) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

CardCommand subclass: #RevealResponseCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!RevealResponseCommand commentStamp: 'JohnParker 7/29/2012 23:28' prior: 0!
A RevealResponseCommand is a command object that sends the revealResponse: cardClass message to the deck specified by deckNumber!

!RevealResponseCommand methodsFor: 'run' stamp: 'JohnParker 7/27/2012 20:34'!
do: board
	[(board deck: deckNumber) revealResponse: cardClass] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].! !

CardCommand subclass: #TreasureCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A TreasureCommand is a command object that sends the playTreasure: cardClass message to the deck specified by deckNumber!

!TreasureCommand methodsFor: 'run' stamp: 'JohnParker 7/27/2012 20:34'!
do: board
	[(board deck: deckNumber) playTreasure: cardClass] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	
	! !

!Command class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber
	^self new deck: aNumber.! !

!Command methodsFor: 'accessing' stamp: 'JohnParker 7/30/2012 12:57'!
deck
	^deckNumber! !

!Command methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:21'!
deck: aNumber
	deckNumber := aNumber.
	! !

!Command methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:01'!
do: board
	self SubclassResponsibility.! !

Command subclass: #EndTurnCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!EndTurnCommand commentStamp: 'JohnParker 7/4/2012 15:04' prior: 0!
An EndTurnCommand is a command object that sends the endTurn message to the deck specified by deckNumber
!

!EndTurnCommand methodsFor: 'run' stamp: 'JohnParker 7/27/2012 20:34'!
do: board
	[(board deck: deckNumber) cleanup] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].! !

Command subclass: #ResolveActionCommand
	instanceVariableNames: 'args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveActionCommand commentStamp: 'JohnParker 7/29/2012 23:26' prior: 0!
A ResolveActionCommand is a command sent by a player to inform their deck of the way in which they would like to resolve the action in play.  Should be sent in response to a #resolveAction: event from their Deck.

Instance Variables
	args:		Dictionary

argVector
	- The dictionary of arguments to be passed.  May vary from card to card.
!

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/6/2012 00:19'!
deck: aNumber argVector: anArgVector
	^self new deck: aNumber argVector: anArgVector! !

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 21:32'!
deck: aNumber args: aDict
	^self new deck: aNumber args: aDict! !

!ResolveActionCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 21:32'!
deck: aNumber args: aDict
	deckNumber := aNumber.
	args := aDict.! !

!ResolveActionCommand methodsFor: 'run' stamp: 'MichaelOSullivan 7/27/2012 23:15'!
do: board
	[(board deck: deckNumber) resolveAction: args] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

Command subclass: #ResolveAttackCommand
	instanceVariableNames: 'args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveAttackCommand commentStamp: 'JohnParker 7/29/2012 23:27' prior: 0!
A ResolveAttackCommand is a command sent by a player to inform their deck of the way in which they would like to resolve an attack against themy.  Should be sent in response to a #attack: event.

Instance Variables
	args:		Dictionary

argVector
	- The dictionary of arguments to be passed.  May vary from card to card.
!

!ResolveAttackCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 12:53'!
deck: aNumber args: aDict
	^self new deck: aNumber args: aDict! !

!ResolveAttackCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 12:54'!
deck: aNumber args: aDict
	deckNumber := aNumber.
	args := aDict.! !

!ResolveAttackCommand methodsFor: 'run' stamp: 'JohnParker 7/31/2012 12:16'!
do: board
	[(board deck: deckNumber) resolveAttack: args] on: DominionError do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

Object subclass: #Deck
	instanceVariableNames: 'deckNumber board areas actions buys phase money resolved attackedBy turnNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Deck commentStamp: 'JohnParker 7/29/2012 23:24' prior: 0!
A Deck represents all the cards in a Dominion Player's possesion and that player's current turn state.  A card can either be in the draw, discard, hand, play, or reveal areas.  It can only be in one.  A card can only be removed from the deck by being trashed, which is to say it is moved to the trash area of the board.

Instance Variables
	areas:		Dictionary: Symbol -> CardArea
	actions: 	Number
	buys: 		Number
	money: 	Number
	phase: 		Symbol
	resolving:	Boolean
	attackedBy: Card
	turnNumber: Number
	
areas
	- This is a dictionary containing all of the places a card in a players posession may be.  Only the following symbols are allowed as keys:		
	#discard
		- The players discard pile.

	#draw
		- The players draw pile.  When this pile is empty the discard pile is shuffled and becomes the new draw pile.

	#hand
		- The player's current hand.  Drawn as a set of 5 cards during the cleanup phase.
	
	#play
		- The play area.  Played cards are placed in this area until the cleanup phase.
	
	#reveal
		- A list of cards currently being "revealed" by the player.  Used by cards that require a player to reveal cards from their deck.
	
	#turnNumber
		- The number of turns a player has taken including this one.

actions
	- The number of actions remaining to be played.

buys
	- The number of buys a player can still make.

money
	- The amount of money the player has this turn due to playing treasure or action cards that give extra money.

phase
	- The phase of the current Turn.  Can be one of:
		#Action 	The player can play actions, treasure, buy cards, or end their turn during this phase
		#Treasure 	The player can play treasure, buy cards, or end their turn during this phase
		#Buy 		The player can buy cards or end their turn during this phase
		#None 		It is not currently the players turn.  Any action will generate an error.

resolving
	- True if the deck is waiting for the player to resolve the last card played.

attackedBy:
	- If not nil, this holds the card that is currently attacking the player.
!

!Deck class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 11:43'!
number: aNumber board: aBoard
	^self new number: aNumber board: aBoard.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
actions
	^actions! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/25/2012 22:00'!
addActions: aNumber
	actions := actions + aNumber.
	self triggerEvent: #deckTurnState: withArguments: {deckNumber}.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/25/2012 22:00'!
addBuys: aNumber
	buys := buys + aNumber.
	self triggerEvent: #deckTurnState: withArguments: {deckNumber}.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/25/2012 22:00'!
addMoney: aNumber
	money := money + aNumber.
	self triggerEvent: #deckTurnState: withArguments: {deckNumber}.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
allCards
	|all|
	all := OrderedCollection new.
	areas do: [:each | all addAll: each].
	^all.! !

!Deck methodsFor: 'attacks' stamp: 'JohnParker 7/30/2012 00:07'!
attack: aCard
	(aCard isNil and: attackedBy isNil not) ifTrue: [attackedBy cancelAttack: deckNumber board: board].
	attackedBy := aCard.
	aCard isNil ifFalse: [self triggerEvent: #attack: withArguments: {aCard class}. ]! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/29/2012 20:47'!
buy: aCardClass
	(board currentDeck = self) ifFalse: [DominionError new signal:  'You can only buy cards on your turn.'].
	(aCardClass cost > money) ifTrue: [DominionError new signal: 'You do not have enough money to purchase a ' , aCardClass cardName].
	(buys = 0) ifTrue: [self cleanup. DominionError new signal: 'You are out of buys.'].
	resolved ifFalse: [DominionError new signal: 'You can not buy a card while resolving another card.'].
	
	(aCardClass buy: board) ifTrue: [
		self addBuys: -1.
		self addMoney: (aCardClass cost) negated.
		(buys = 0) ifTrue: [self cleanup].
	] ifFalse: [DominionError new signal: 'Could not buy a ' , aCardClass cardName]
	
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
buys
	^buys! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 20:54'!
card: aCardClass from: area1 to: area2
	|card|
	card := (areas at: area1) detect: [:each | each class = aCardClass] ifNone: [^false].
	(areas at: area1) remove: card ifAbsent: [^false].
	(areas at: area2) addLast: card.
	^true.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:11'!
cardResolved: val
	resolved := val! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/29/2012 20:47'!
cleanup
	(board currentDeck = self) ifFalse: [DominionError new signal: 'You can only cleanup on your turn.'].
	resolved ifFalse: [DominionError new signal: 'You can not end your turn while resolving a card.'].
	self default.
	board nextDeck.
	! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/25/2012 22:02'!
default
	actions := 1.
	buys := 1.
	money := 0.
	phase := #None.
	resolved := true.
	self triggerEvent: #deckTurnState: withArguments: {deckNumber}.
	
	(areas at: #discard) addAllLast: self play.
	self play removeAll.
	
	(areas at: #discard) addAllLast: self hand.
	self hand removeAll.
	
	1 to: 5 do: [:a | self drawCardTo: #hand].! !

!Deck methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/20/2012 09:45'!
discardCount
	^ (areas at: #discard) size! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 8/2/2012 12:42'!
discardTop
	^ (areas at: #discard) last class! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 20:56'!
drawCard
	| card draw discard|
	"Reshuffle the discard into the draw if the draw is empty"
	draw := areas at: #draw.
	discard := areas at: #discard.
	
	(draw size = 0) ifTrue: [
		draw addAllLast: discard. 
		discard removeAll. 
		draw shuffle.
		self triggerEvent: #deckShuffled: withArguments: {deckNumber}.
	].
	"If the draw is still empty, we have no cards to draw so do nothing"
	(draw size = 0) ifFalse: [
		card := draw last.
		draw removeLast.
	].
	self  triggerEvent: #cardDrawn: withArguments: {card class}.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/30/2012 19:15'!
drawCardTo: anArea
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		(areas at: anArea) add: card.
		self triggerMoveEvent: card class from: #draw to: anArea.
	].
	
	^card.
	! !

!Deck methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/20/2012 09:45'!
drawCount
	^ (areas at: #draw) size! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:13'!
drawSize
	^(areas at: #draw) size.! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/29/2012 11:26'!
gainCard: aCardClass to: anArea
	|card|
	card := board takeCard: aCardClass deck: deckNumber.
	card isNil ifFalse: [
		(areas at: anArea) addLast: card.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. #board. anArea}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
hand
	^areas at: #hand! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
handContains: aCardClass
	^(self hand detect: [:each | each class = aCardClass] ifNone: [nil]) isNil not.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:49'!
handContainsAll: aCardClassVector
	|set|
	set := aCardClassVector asSet.
	^set = (set select: [:testCardClass | 
		(self hand count: [:handCard | handCard class = testCardClass]) >=  
			(aCardClassVector count: [:each | each class = testCardClass])
	]).
	! !

!Deck methodsFor: 'accessing' stamp: 'ChrisLau 7/22/2012 22:59'!
handCount: aCardClass
	^(self hand count: [:each | each class = aCardClass]).
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:22'!
money
	^money! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/17/2012 21:57'!
moveCard: aCardClass from: area1 to: area2
	|card|
	
	((areas at: area1) visibility = #hidden) ifTrue: [^false].
	card := (areas at: area1) detect: [:each | each class = aCardClass] ifNone: [^false].
	(areas at: area1) remove: card ifAbsent: [^false].
	(areas at: area2) addLast: card.
	
	self triggerMoveEvent: aCardClass from: area1 to: area2.
	^true.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 8/11/2012 02:13'!
moveCardResolve: aCardClass from: fromArea to: toArea
	(board currentDeck = self) ifFalse: [DominionError new signal: 'You can only resolve action cards on your turn.'].
	resolved ifTrue: [DominionError new signal: 'There is no action to resolve right now.'].
	
	self play last moveCardResolve: aCardClass from: fromArea to: toArea board: board.
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 8/12/2012 14:51'!
moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber
	(board currentDeck = self) ifFalse: [DominionError new signal: 'You can only resolve action cards on your turn.'].
	resolved ifTrue: [DominionError new signal: 'There is no action to resolve right now.'].
	
	self play last moveCardResolve: aCardClass from: fromArea to: toArea deck: aNumber board: board.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 20:29'!
number
	^deckNumber! !

!Deck methodsFor: 'initialize-release' stamp: 'JohnParker 7/30/2012 12:49'!
number: aNumber board: aBoard
	deckNumber := aNumber.
	board := aBoard.
	turnNumber := 0.
	areas := Dictionary new.
	areas at: #discard put: (CardArea visibility: #global).
	areas at: #draw put: (CardArea visibility: #hidden).
	areas at: #hand put: (CardArea visibility: #player).
	areas at: #play put: (CardArea visibility: #global).
	areas at: #reveal put: (CardArea visibility: #global).
	
	1 to: 7 do: [:a | self gainCard: Copper to: #discard].
	1 to: 3 do: [:a | self gainCard: Estate to: #discard].
	
	self default.
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 12:45'!
phase
	^phase! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:49'!
play
	^areas at: #play! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/29/2012 20:47'!
playAction: aCardClass
	| card |
	(board currentDeck = self) ifFalse: [DominionError new signal: 'You can only play action cards on your turn.'].
	(aCardClass inheritsFrom: ActionCard) ifFalse: [DominionError new signal: aCardClass cardName , ' is not an action card.'].
	(phase = #Treasure) ifTrue: [DominionError new signal: 'You can not play an action during the treasure phase.'].
	(phase = #Buy) ifTrue: [DominionError new signal: 'You can not play an action during the buy phase.'].
	resolved ifFalse: [DominionError new signal: 'You can not play an action while resolving another card.'].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [DominionError new signal: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	self addActions: -1.
	
	card play: board.
	
	(actions = 0) ifTrue: [phase := #Treasure].! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/29/2012 20:48'!
playTreasure: aCardClass
	| card |
	(board currentDeck = self) ifFalse: [DominionError new signal: 'You can only play treasure cards on your turn.'].
	(aCardClass inheritsFrom: TreasureCard) ifFalse: [DominionError new signal: aCardClass cardName , ' is not a treasure card.'].
	(phase = #Buy) ifTrue: [DominionError new signal: 'You can not play a treasure during the buy phase.'].
	resolved ifFalse: [DominionError new signal: 'You can not play a treasure while resolving another card.'].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [DominionError new signal: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	card play: board.
	phase := #Treasure.
	
	self triggerEvent: #cardPlayed withArguments: {deckNumber. aCardClass}.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/29/2012 20:48'!
resolveAction: argVector
	(board currentDeck = self) ifFalse: [DominionError new signal: 'You can only resolve action cards on your turn.'].
	resolved ifTrue: [DominionError new signal: 'There is no action to resolve right now.'].
	
	self play last resolve: argVector board: board.
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/27/2012 20:32'!
resolveAttack: args
	(board currentDeck = self) ifTrue: [DominionError new signal: 'You can not be attacked on your turn.'].
	attackedBy ifNil: [^self].
	
	attackedBy resolveAttack: args board: board deck: self.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 8/11/2012 02:25'!
resolving 
	^resolved not ! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/19/2012 20:08'!
reveal
	^areas at: #reveal! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/27/2012 20:32'!
revealResponse: aCardClass
	| card |
	(board currentDeck = self) ifTrue: [DominionError new signal: 'You can not be attacked on your turn.'].
	attackedBy ifNil: [^self].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [DominionError new signal: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	self moveCard: card class from: #hand to: #reveal.
	self moveCard: card class from: #reveal to: #hand.
	card revealResponse: board deck: self.
	
	
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/29/2012 19:34'!
startTurn
	phase := #Action.
	turnNumber := turnNumber + 1.
	self triggerEvent: #turnStarted: withArguments: {deckNumber} .! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/22/2012 21:34'!
takeTrashTopTo: anArea
	|card|
	card := board takeTrashTop.
	
	card isNil ifFalse: [
		(areas at: anArea) addLast: card.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. card class. #trash. anArea}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/28/2012 15:57'!
trashCard: aCardClass from: anArea
	|card|
	card := (areas at: anArea) detect: [:each | each class = aCardClass] ifNone: [nil].
	card isNil ifFalse: [
		(areas at: anArea) remove: card.
		board trashCard: card deck: deckNumber.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. anArea. #trash}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 22:10'!
triggerMoveEvent: aCardClass from: area1 to: area2
	(((areas at: area1) visibility = #global) or: ((areas at: area2) visibility = #global)) ifTrue: [
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. area1. area2}.
	] ifFalse: [
		self triggerEvent: #deck:cardMovedPlayer:from:to: withArguments: {deckNumber. aCardClass. area1. area2}.
	].! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/29/2012 11:31'!
turns
	^turnNumber.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 8/11/2012 10:34'!
victoryPoints
	(board gameOver) ifFalse: [DominionError new signal: 'No peeking!! The game is not over yet!!'].
	^self allCards inject: 0 into: [:points :card |
		((card class inheritsFrom: VictoryCard) or: (card class inheritsFrom: CurseCard)) ifTrue: [points + (card value: board deck: self)] ifFalse: [points]
	]! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:58'!
z_addCardToHand: aCardClass
	self hand add: aCardClass basicNew! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 13:13'!
z_areas
	^areas! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:59'!
z_discard
	^(areas at: #discard)! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:59'!
z_draw
	^(areas at: #draw)! !

Object subclass: #KingdomRenderer
	instanceVariableNames: 'startingPoint currentPosition remaining rect row kingdom rowWidth rowHeight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!KingdomRenderer commentStamp: 'MichaelOSullivan 7/30/2012 17:41' prior: 0!
KingdomRenderer is a method object that draws the CardMorphs contained within a Kingdom.!

!KingdomRenderer methodsFor: 'computing' stamp: 'JohnParker 8/11/2012 11:00'!
compute
	startingPoint := kingdom innerBounds origin translateBy: 5 @ 5.
	kingdom title position: startingPoint.
	currentPosition := kingdom title position translateBy: 0 @ (kingdom title height + 20).
	
	1 to: 6 do: [:rowNum | 
		|start cardRow| 
		start:= (((rowNum - 1) * 3)+1). 
		cardRow := kingdom cards copyFrom: start to: ((start + 2) min: 17).
		self processRow: cardRow.
	].
	
	rect := Rectangle 	origin: (kingdom innerBounds origin)
				    		corner: (currentPosition translateBy: rowWidth @ 0).
	kingdom height: (currentPosition y - kingdom top).
	kingdom width: rowWidth.! !

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/17/2012 23:00'!
doCarriageFeedLineReturn
	"moves the current position down a row and back to the far left of the area"

	currentPosition := currentPosition translateBy: (startingPoint x - currentPosition x) @ (rowHeight + 20)! !

!KingdomRenderer methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/18/2012 22:29'!
initializeWith: aKingdom
	kingdom := aKingdom.
	rowWidth := 0.
	rowHeight := 0! !

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/18/2012 22:48'!
processRow: aCollection
	aCollection do: [:each |  self renderCard: each].
	rowWidth := rowWidth max: currentPosition x.
	self doCarriageFeedLineReturn! !

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/17/2012 23:21'!
renderCard: aCard

	aCard position: currentPosition.
	currentPosition  := currentPosition translateBy: (aCard width + 20) @ 0.
	rowHeight := rowHeight max: aCard height! !

Object subclass: #TestPlayer
	instanceVariableNames: 'board deckNumber errorCommand errorString events'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!TestPlayer commentStamp: 'JohnParker 7/10/2012 19:56' prior: 0!
A Player is an abstract class that represents a player.  In this design, a player controls the corresponding deck, sending commands to it in order to play and respond to cards.  A Player also listens for events from Board and Deck which inform it of changes to the game state.  The player may also display the current state of the board if it is a human player.

Instance Variables
	deckNumber:		Number

deckNumber
	- The number of the deck this player controls.
!

TestPlayer subclass: #BigMoney
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-AI'!

!BigMoney methodsFor: 'events' stamp: 'JohnParker 7/29/2012 19:47'!
turnStarted:  aNumber
	|deck|
	(aNumber = deckNumber) ifFalse: [^self].
	
	deck := board deck: deckNumber.
	(deck hand select: [:each | each class inheritsFrom: TreasureCard]) do: [:each |
		board execute: (TreasureCommand deck: deckNumber card: each class).
	].

	(deck money >= 8) ifTrue: [
		board execute: (BuyCommand deck: deckNumber card: Province).
		^self.
	].
	
	((deck money >= 6) and: ((board cardsRemaining: Province) > 3)) ifTrue: [
		board execute: (BuyCommand deck: deckNumber card: Gold).
		^self.
	].
	
	(deck money >= 5) ifTrue: [
		board execute: (BuyCommand deck: deckNumber card: Duchy).
		^self.
	].
	
	(deck money >= 3) ifTrue: [
		board execute: (BuyCommand deck: deckNumber card: Silver).
		^self.
	].
	
	board execute: (EndTurnCommand deck: deckNumber).! !

!TestPlayer class methodsFor: 'instance creation' stamp: 'JohnParker 7/27/2012 18:05'!
board: aBoard deck: aNumber
	self new board: aBoard deck: aNumber.! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/28/2012 20:20'!
attack: aCardClass
	|card deck|
	"This event is called when we are attacked.  We must respond with the correct ResolveAttackCommand"
	
	deck := board deck: deckNumber.
	events addLast: (Message selector: #attack: arguments: {aCardClass}).
	
	"If we have a response card (like the moat) to play in order to block the attack, then reveal it"
	card := (board deck: deckNumber) hand detect: [:each | each class inheritsFrom: ResponseCard] ifNone: [nil].
	card isNil ifFalse: [board execute: (RevealResponseCommand deck: deckNumber card: card class)].
	
	(aCardClass = Bureaucrat) ifTrue: [
		|command args|
		card := (board deck: deckNumber) hand detect: [:each | each class inheritsFrom: VictoryCard] ifNone: [nil].
		
		args := Dictionary new.
		args at: #bureaucratCard put: nil.
		(card isNil) ifFalse: [
			args at: #bureaucratCard put: card class.
		].
		command := ResolveAttackCommand deck: deckNumber args: args.
		
		board execute: command.
	].

	(aCardClass = Militia) ifTrue: [
		|dict|
		
		dict := Dictionary new.
		dict at: #discards put: OrderedCollection new.
		1 to: (deck hand size - 3) do: [:num |
			(dict at: #discards) add: (deck hand at: num) class.
		]. 
		board execute: (ResolveAttackCommand deck: deckNumber args: dict)
	].
	
	((aCardClass = Witch) or: ((aCardClass = Spy) or: (aCardClass = Thief))) ifTrue: [
		board execute: (ResolveAttackCommand deck: deckNumber args: Dictionary new)
	]! !

!TestPlayer methodsFor: 'setup' stamp: 'JohnParker 7/29/2012 19:50'!
board: aBoard deck: aNumber
	board := aBoard.
	deckNumber := aNumber.
	events := OrderedCollection new.
	
	board addObserver: self at: aNumber.
	board addObserver: self.! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:25'!
clearError
	errorCommand := nil.
	errorString := nil.! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/19/2012 20:00'!
command: aCommand error: aString
	errorCommand := aCommand.
	errorString := aString.
	events addLast: (Message selector: #command:error: arguments: {aCommand. aString}).! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/29/2012 19:45'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	events addLast: aMessage.
	! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:24'!
errorCommand
	^errorCommand! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:24'!
errorString
	^errorString! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/28/2012 20:08'!
events
	^events! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 8/12/2012 21:21'!
resolveAction: aCardClass deck: aNumber
	|dict|
	"This event is called when an attack we played needs a resolution for each player we attacked.  aNumber is the deck number of the attacked player we are resolving."
	events addLast: (Message selector: #resolveAction:deck: arguments: {aCardClass. aNumber}).
	
	dict := Dictionary new.
	
	(aCardClass = Spy) ifTrue: [
		|discard|
		
		discard := (board deck: aNumber) reveal last class inheritsFrom: TreasureCard.
		(deckNumber = aNumber) ifTrue: [discard := discard not].
		
		discard ifTrue: [
			board execute: (MoveCardResolveCommand deck: aNumber card: (board deck: aNumber) reveal last class from: #reveal to: #discard).
		] ifFalse: [
			board execute: (MoveCardResolveCommand deck: aNumber card: (board deck: aNumber) reveal last class from: #reveal to: #hand).
		].
	].

	(aCardClass = Thief) ifTrue: [
		|trash|
		
		trash := (board deck: aNumber) reveal detect: [:each | each class inheritsFrom: TreasureCard].
		
		board execute: (MoveCardResolveCommand deck: aNumber card: trash class from: #reveal to: #discard).
	].! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 8/12/2012 14:30'!
resolveAction: aCardClass message: aString
	|deck|
	"This event is called when this player played an action that needs resolution.  We must respond with the correct ResolveActionCommand"
	deck := board deck: deckNumber.
	events addLast: (Message selector: #resolveAction: arguments: {aCardClass}).
	
	(aCardClass = Cellar) ifTrue: [
		| victoryCards |
		victoryCards := (board deck: deckNumber) hand select: [:each | each class inheritsFrom: VictoryCard] thenCollect: [:each | each class].
		
		victoryCards do: [:each | board execute: (MoveCardResolveCommand deck: deckNumber card: each from: #hand to: #discard)].
		board execute: (ResolveActionCommand deck: deckNumber args: nil).
	].

	(aCardClass = Chapel) ifTrue: [
		| trashCards |
		trashCards := (board deck: deckNumber) hand select: [:each | (each class = Copper) or: (each class = Estate)] thenCollect: [:each | each class].
		trashCards removeLast: (trashCards size - 4).
		
		trashCards do: [:each | board execute: (MoveCardResolveCommand deck: deckNumber card: each from: #hand to: #trash)].
		board execute: (ResolveActionCommand deck: deckNumber args: nil).
	].

	(aCardClass = Chancellor) ifTrue: [
		board execute: (MoveCardResolveCommand deck: deckNumber card: Card from: #draw to: #discard)
	].
	
	(aCardClass = Library) ifTrue: [
		board execute: (MoveCardResolveCommand deck: deckNumber card: deck hand last class from: #hand to: #reveal)
	].
	
	(aCardClass = Feast) ifTrue: [
	 	board execute: (MoveCardResolveCommand deck: deckNumber card: Silver from: #board to: #discard)
	].
	
	(aCardClass = Workshop) ifTrue: [
		board execute: (MoveCardResolveCommand deck: deckNumber card: Silver from: #board to: #discard)
	].
	
	(aCardClass = Mine) ifTrue: [
		| treasureCards trashCard gainCard |
		treasureCards := (board deck: deckNumber) hand select: [:each | each class inheritsFrom: TreasureCard] thenCollect: [:each | each class].
		trashCard := treasureCards detectMin: [:each | each cost].
		gainCard := {Silver. Gold} detect: [:each | each cost = (trashCard cost + 3)].
		
		board execute: (MoveCardResolveCommand deck: deckNumber card: trashCard from: #hand to: #trash).
		board execute: (MoveCardResolveCommand deck: deckNumber card: gainCard from: #board to: #hand).
	].

	(aCardClass = Remodel) ifTrue: [
		| trashCard gainCard |
		trashCard := Estate.
		gainCard := Silver.
		
		board execute: (MoveCardResolveCommand deck: deckNumber card: trashCard from: #hand to: #trash).
		board execute: (MoveCardResolveCommand deck: deckNumber card: gainCard from: #board to: #discard).
	].
	
	(aCardClass = ThroneRoom) ifTrue: [
		| playCard |
		playCard := deck hand detect: [:each | each class inheritsFrom: ActionCard].
		board execute: (MoveCardResolveCommand deck: deckNumber card: playCard class from: #hand to: #play).
	].! !
